 /* +---------------------------------------------------------------- */
 /*                                                                   */
 /*  Microsoft Windows                                                */
 /*  Copyright (C) Microsoft Corporation, 1993-1997.                  */
 /*                                                                   */
 /*  File:       accctrl.cpy                                          */
 /*                                                                   */
 /*  Contents:   common includes for new style Win32 Access Control   */
 /*              APIs                                                 */
 /*                                                                   */
 /* ----------------------------------------------------------------- */
 %dcl #_ACCESS_CONTROL__ char ext;
 %if #_ACCESS_CONTROL__ = '' %then  %do;
 %#_ACCESS_CONTROL__ = 'Y';

 %dcl UNICODE char ext;

 %dcl AccFree char ext;
 %AccFree = 'LocalFree';

 /* Definition: TRUSTEE_TYPE */
 /* This enumerated type specifies the type of trustee account for    */
 /* the trustee returned by the API described in this document.       */
 /* TRUSTEE_IS_UNKNOWN - The trustee is an unknown, but not           */
 /*                      necessarily invalid type. This field is not  */
 /*                      validated on input to the APIs               */
 /*                      that take Trustees.                          */
 /* TRUSTEE_IS_USER      The trustee account is a user account.       */
 /* TRUSTEE_IS_GROUP     The trustee account is a group account.      */

 define ordinal
   TRUSTEE_TYPE
     ( TRUSTEE_IS_UNKNOWN,
       TRUSTEE_IS_USER,
       TRUSTEE_IS_GROUP,
       TRUSTEE_IS_DOMAIN,
       TRUSTEE_IS_ALIAS,
       TRUSTEE_IS_WELL_KNOWN_GROUP,
       TRUSTEE_IS_DELETED,
       TRUSTEE_IS_INVALID
     ) precision(31);


 /* Definition: TRUSTEE_FORM                                          */
 /* This enumerated type specifies the form the trustee identifier is */
 /* in for a particular trustee.                                      */
 /* TRUSTEE_IS_SID       The trustee is identified with a SID rather  */
 /*                      than with a name.                            */
 /* TRUSTEE_IS_NAME      The trustee is identified with a name.       */

 define ordinal
   TRUSTEE_FORM
     ( TRUSTEE_IS_SID,
       TRUSTEE_IS_NAME,
       TRUSTEE_BAD_FORM
     ) precision(31);



 /* Definition: MULTIPLE_TRUSTEE_OPERATION                            */
 /* If the trustee is a multiple trustee, this enumerated type        */
 /* specifies the type.                                               */
 /* TRUSTEE_IS_IMPERSONATE   The trustee is an impersonate trustee    */
 /*                          and the multiple trustee field in the    */
 /*                          trustee points to another trustee that   */
 /*                          is a trustee for the server that will be */
 /*                          doing the impersonation.                 */

 define ordinal
   MULTIPLE_TRUSTEE_OPERATION
     ( NO_MULTIPLE_TRUSTEE,
       TRUSTEE_IS_IMPERSONATE
     ) precision(31);


 /* Definition: TRUSTEE                                               */
 /* This structure is used to pass account information into and out   */
 /* of the system using the API defined in this document.             */
 /* PMultipleTrustee - if NON-NULL, points to another trustee         */
 /*                    structure, as defined by the multiple trustee  */
 /*                    operation field.                               */
 /* MultipleTrusteeOperation - Defines the multiple trustee           */
 /*                            operation/type.                        */
 /* TrusteeForm      - defines if the trustee is defined by name or   */
 /*                    SID.                                           */
 /* TrusteeType      - defines if the trustee type is unknown, a user */
 /*                    or a group.                                    */
 /* PwcsName         - points to the trustee name or the trustee SID. */

 define structure
   1 TRUSTEE_A,
       2 pMultipleTrustee         handle TRUSTEE_A,
       2 MultipleTrusteeOperation type MULTIPLE_TRUSTEE_OPERATION,
       2 TrusteeForm              type TRUSTEE_FORM,
       2 TrusteeType              type TRUSTEE_TYPE,
       2 ptstrName                type LPSTR;
 define alias PTRUSTEE_A handle TRUSTEE_A;
 define alias  TRUSTEEA  type   TRUSTEE_A;
 define alias PTRUSTEEA  handle TRUSTEE_A;

 define structure
   1 TRUSTEE_W,
       2 pMultipleTrustee         handle TRUSTEE_W,
       2 MultipleTrusteeOperation type MULTIPLE_TRUSTEE_OPERATION,
       2 TrusteeForm              type TRUSTEE_FORM,
       2 TrusteeType              type TRUSTEE_TYPE,
       2 ptstrName                type LPWSTR;
 define alias PTRUSTEE_W handle TRUSTEE_W;
 define alias  TRUSTEEW  type   TRUSTEE_W;
 define alias PTRUSTEEW  handle TRUSTEE_W;

 %if UNICODE ^= '' %then %do;
 define alias  TRUSTEE_ type TRUSTEE_W;
 define alias PTRUSTEE_ type PTRUSTEE_W;
 define alias  TRUSTEE  type TRUSTEEW;
 define alias PTRUSTEE  type PTRUSTEEW;
 %end; %else %do;
 define alias  TRUSTEE_ type TRUSTEE_A;
 define alias PTRUSTEE_ type PTRUSTEE_A;
 define alias  TRUSTEE  type TRUSTEEA;
 define alias PTRUSTEE  type PTRUSTEEA;
 %end; /* UNICODE */



 /* Definition: ACCESS_MODE                                           */
 /* This enumerated type specifies how permissions are (requested)/to */
 /* be applied for the trustee by the access control entry.           */
 /* On input this field can by any of the values, although it is not  */
 /* meaningful to mix access control and audit control entries.       */
 /* On output this field will be either SET_ACCESS, DENY_ACCESS,      */
 /* SET_AUDIT_SUCCESS, SET_AUDIT_FAILURE.                             */
 /* The following descriptions define how this type effects an        */
 /* explicit access request to apply access permissions to an object. */
 /* GRANT_ACCESS - The trustee will have at least the requested       */
 /*                permissions upon successful completion of the      */
 /*                command. (If the trustee has additional            */
 /*                permissions they will not be removed).             */
 /* SET_ACCESS        - The trustee will have exactly the requested   */
 /*                     permissions upon successful completion of the */
 /*                     command.                                      */
 /* DENY_ACCESS       - The trustee will be denied the specified      */
 /*                     permissions.                                  */
 /* REVOKE_ACCESS     - Any explicit access rights the trustee has    */
 /*                     will be revoked.                              */
 /* SET_AUDIT_SUCCESS - The trustee will be audited for successful    */
 /*                     opens of the object using the requested       */
 /*                     permissions.                                  */
 /* SET_AUDIT_FAILURE - The trustee will be audited for failed opens  */
 /*                     of the object using the requested permissions.*/

 define ordinal
   ACCESS_MODE
     ( NOT_USED_ACCESS value(0),
       GRANT_ACCESS,
       SET_ACCESS,
       DENY_ACCESS,
       REVOKE_ACCESS,
       SET_AUDIT_SUCCESS,
       SET_AUDIT_FAILURE
     ) precision(31);


 /* Definition: Inheritance flags                                     */
 /* These bit masks are provided to allow simple application of       */
 /* inheritance in explicit access requests on containers.            */
 /* NO_INHERITANCE  The specific access permissions will only be      */
 /*                 applied to the container, and will not be         */
 /*                 inherited by objects created within the container.*/
 /* SUB_CONTAINERS_ONLY_INHERIT  The specific access permissions will */
 /*                              be inherited and applied to sub      */
 /*                              containers created within the        */
 /*                              container, and will be applied to    */
 /*                              the container itself.                */
 /* SUB_OBJECTS_ONLY_INHERIT     The specific access permissions will */
 /*                              only be inherited by objects created */
 /*                              within the specific container.       */
 /*                              The access permissions will not be   */
 /*                              applied to the container itself.     */
 /* SUB_CONTAINERS_AND_OBJECTS_INHERIT                                */
 /*                              The specific access permissions will */
 /*                              be inherited by containers created   */
 /*                              within the specific container, will  */
 /*                              be applied to objects created within */
 /*                              the container, but will not be       */
 /*                              applied to the container itself.     */

 dcl NO_INHERITANCE                      type DWORD value('0'xu);
 dcl SUB_OBJECTS_ONLY_INHERIT            type DWORD value('1'xu);
 dcl SUB_CONTAINERS_ONLY_INHERIT         type DWORD value('2'xu);
 dcl SUB_CONTAINERS_AND_OBJECTS_INHERIT  type DWORD value('3'xu);
 dcl INHERIT_NO_PROPAGATE                type DWORD value('4'xu);
 dcl INHERIT_ONLY                        type DWORD value('8'xu);


 /* Informational bit that is returned */

 dcl INHERITED_ACCESS_ENTRY              type DWORD value('10'xu);


 /* Informational bit that tells where a node was inherited from. */
 /* Valid only for NT 5 APIs                                      */

 dcl INHERITED_PARENT                    type DWORD value('10000000'xu);
 dcl INHERITED_GRANDPARENT               type DWORD value('20000000'xu);


 /* Definition:                                                       */
 /* This enumerated type defines the objects supported by the get/set */
 /* API within this document. See section 3.1, Object Types for a     */
 /* detailed definition of the supported object types, and their name */
 /* formats.                                                          */

 define ordinal
   SE_OBJECT_TYPE
     ( SE_UNKNOWN_OBJECT_TYPE value(0),
       SE_FILE_OBJECT,
       SE_SERVICE,
       SE_PRINTER,
       SE_REGISTRY_KEY,
       SE_LMSHARE,
       SE_KERNEL_OBJECT,
       SE_WINDOW_OBJECT,
       SE_DS_OBJECT,
       SE_DS_OBJECT_ALL,
       SE_PROVIDER_DEFINED_OBJECT,
       SE_WMIGUID_OBJECT
     ) precision(31);


 /* Definition: EXPLICIT_ACCESS                                       */
 /* This structure is used to pass access control entry information   */
 /* into and out of the system using the API defined in this document.*/
 /* grfAccessPermissions - This contains the access permissions to    */
 /*                        assign for the trustee.  It is in the form */
 /*                        of an NT access mask.                      */
 /* grfAccessMode        - This field defines how the permissions are */
 /*                        to be applied for the trustee.             */
 /* grfInheritance       - For containers, this field defines how the */
 /*                        access control entry is/(is requested) to  */
 /*                        be inherited on objects/sub-containers     */
 /*                        created within the container.              */
 /* Trustee              - This field contains the definition of the  */
 /*                        trustee account the explicit access        */
 /*                        applies to.                                */

 define structure
   1 EXPLICIT_ACCESS_A,
     2 grfAccessPermissions type DWORD,
     2 grfAccessMode        type ACCESS_MODE,
     2 grfInheritance       type DWORD,
     2 Trustee              type TRUSTEE_A;
 define alias PEXPLICIT_ACCESS_A handle EXPLICIT_ACCESS_A;
 define alias  EXPLICIT_ACCESSA  type   EXPLICIT_ACCESS_A;
 define alias PEXPLICIT_ACCESSA  handle EXPLICIT_ACCESS_A;

 define structure
   1 EXPLICIT_ACCESS_W,
     2 grfAccessPermissions type DWORD,
     2 grfAccessMode        type ACCESS_MODE,
     2 grfInheritance       type DWORD,
     2 Trustee              type TRUSTEE_W;
 define alias PEXPLICIT_ACCESS_W handle EXPLICIT_ACCESS_W;
 define alias  EXPLICIT_ACCESSW  type   EXPLICIT_ACCESS_W;
 define alias PEXPLICIT_ACCESSW  handle EXPLICIT_ACCESS_W;

 %if UNICODE ^= '' %then %do;
 define alias  EXPLICIT_ACCESS_ type EXPLICIT_ACCESS_W;
 define alias PEXPLICIT_ACCESS_ type PEXPLICIT_ACCESS_W;
 define alias  EXPLICIT_ACCESS  type EXPLICIT_ACCESSW;
 define alias PEXPLICIT_ACCESS  type PEXPLICIT_ACCESSW;
 %end; %else %do;
 define alias  EXPLICIT_ACCESS_ type EXPLICIT_ACCESS_A;
 define alias PEXPLICIT_ACCESS_ type PEXPLICIT_ACCESS_A;
 define alias  EXPLICIT_ACCESS  type EXPLICIT_ACCESSA;
 define alias PEXPLICIT_ACCESS  type PEXPLICIT_ACCESSA;
 %end; /* UNICODE */

 %end; /* __ACCESS_CONTROL__ */

