 /* tk.cpy --                                                         */
 /*                                                                   */
 /* Declarations for Tk-related things that are visible               */
 /* outside of the Tk module itself.                                  */
 /*                                                                   */
 /* Copyright (c) 1989-1994 The Regents of the University of          */
 /*                         California.                               */
 /* Copyright (c) 1994 The Australian National University.            */
 /* Copyright (c) 1994-1994 Sun Microsystems, Inc.                    */
 /* Copyright (c) 1998-2000 by Scriptics Corporation.                 */
 /* Copyright (c) 1998-2000 Mark Yudkin (PL/I translation)            */
 /*                                                                   */
 /* See the file "license.terms" for information on usage and redist- */
 /* ribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.    */
 /*                                                                   */

 %dcl #TK char ext;
 %if #TK = '' %then %do;
 %#TK = 'Y';

 /* When version numbers change here, you must also go into the       */
 /* following files and update the version numbers:                   */
 /*                                                                   */
 /* unix/configure.in                                                 */
 /* win/makefile.bc                                                   */
 /* win/makefile.vc                                                   */
 /* library/tk.tcl                                                    */
 /*                                                                   */
 /* The release level should be 0 for alpha, 1 for beta, and 2 for    */
 /* final/patch.  The release serial value is the number that follows */
 /* the "a", "b", or "p" in the patch level; for example, if the      */
 /* patch level is 4.3b2, TK_RELEASE_SERIAL is 2. It restarts at 1    */
 /* whenever the release level is changed, except for the final       */
 /* release, which should be 0.                                       */
 /*                                                                   */
 /* You may also need to update some of these files when the numbers  */
 /* change for the version of Tcl that this release of Tk is compiled */
 /* against.                                                          */

 %dcl #TCL char ext;
 %if #TCL = '' %then     %do;
   %include tcl;
 %end;

 %dcl TCL_MEM_DEBUG    char ext;
 %dcl #_NO_OLD_CONFIG  char ext;
 %dcl USE_TK_STUBS     char ext;

 dcl TK_MAJOR_VERSION  fixed bin(31) value(8);
 dcl TK_MINOR_VERSION  fixed bin(31) value(3);
 dcl TK_RELEASE_LEVEL  fixed bin(31) value(TCL_FINAL_RELEASE);
 dcl TK_RELEASE_SERIAL fixed bin(31) value(3);

 dcl TK_VERSION        char value('8.3');
 dcl TK_PATCH_LEVEL    char value('8.3.3');

 %dcl #XLIB_H char ext;
 %if #XLIB_H = '' %then %do;
   %include "X11\Xlib.cpy";
 %end;

 /* Decide whether or not to use input methods.  */

 %dcl TK_USE_INPUT_METHODS char ext;
 %dcl XNQueryInputStyle char ext;
 %if XNQueryInputStyle ^= '' %then %do;
   %TK_USE_INPUT_METHODS = 'Y';
 %end;

 /* Dummy types that are used by clients: */

 define structure 1 Tk_BindingTable_;
 define alias Tk_BindingTable   handle Tk_BindingTable_;
 define structure 1 Tk_Canvas_;
 define alias Tk_Canvas         handle Tk_Canvas_;
 define structure 1 Tk_Cursor_;
 define alias Tk_Cursor         handle Tk_Cursor_;
 define structure 1 Tk_ErrorHandler_;
 define alias Tk_ErrorHandler   handle Tk_ErrorHandler_;
 define structure 1 Tk_Font_;
 define alias Tk_Font           handle Tk_Font_;
 define structure 1 Tk_Image_;
 define alias Tk_Image          handle Tk_Image_;
 define structure 1 Tk_ImageMaster_;
 define alias Tk_ImageMaster    handle Tk_ImageMaster_;
 define structure 1 Tk_OptionTable_;
 define alias Tk_OptionTable    handle Tk_OptionTable_;
 define structure 1 Tk_PostscriptInfo_;
 define alias Tk_PostscriptInfo handle Tk_PostscriptInfo_;
 define structure 1 Tk_TextLayout_;
 define alias Tk_TextLayout     handle Tk_TextLayout_;
 define structure 1 Tk_Window_;
 define alias Tk_Window         handle Tk_Window_;
 define structure 1 Tk_3DBorder_;
 define alias Tk_3DBorder       handle Tk_3DBorder_;

 /* Additional types exported to clients. */

 define alias Tk_Uid pointer;

 /* The enum below defines the valid types for Tk configuration       */
 /* options as implemented by Tk_InitOptions, Tk_SetOptions, etc.     */

 define ordinal Tk_OptionType
   ( TK_OPTION_BOOLEAN,
     TK_OPTION_INT,
     TK_OPTION_DOUBLE,
     TK_OPTION_STRING,
     TK_OPTION_STRING_TABLE,
     TK_OPTION_COLOR,
     TK_OPTION_FONT,
     TK_OPTION_BITMAP,
     TK_OPTION_BORDER,
     TK_OPTION_RELIEF,
     TK_OPTION_CURSOR,
     TK_OPTION_JUSTIFY,
     TK_OPTION_ANCHOR,
     TK_OPTION_SYNONYM,
     TK_OPTION_PIXELS,
     TK_OPTION_WINDOW,
     TK_OPTION_END
   ) prec(31);

 /* Structures of the following type are used by widgets to specify   */
 /* their configuration options.  Typically each widget has a static  */
 /* array of these structures, where each element of the array        */
 /* describes a single configuration option.  The array is passed to  */
 /* Tk_CreateOptionTable.                                             */

 define structure
   1 Tk_OptionSpec,
     2 type       type Tk_OptionType,
       /* Type of option, such as TK_OPTION_COLOR, see definitions    */
       /* above. Last option in table must have type TK_OPTION_END.   */
     2 optionName pointer,
       /* Name used to specify option in Tcl commands.                */
     2 dbName     pointer, /* Name for option in option database.     */
     2 dbClass    pointer, /* Class for option in database.           */
     2 defValue   pointer,
       /* Default value for option if not specified in command line,  */
       /* the option database, or the system.                         */
     2 objOffset  fixed bin (31),
       /* Where in record to store a Tcl_Obj * that holds the value   */
       /* of this option, specified as an offset in bytes from the    */
       /* start of the record. Use the Tk_Offset macro to generate    */
       /* values for this.                                            */
       /* -1 means don't store the Tcl_Obj in the record.             */
     2 internalOffset fixed bin (31),
       /* Where in record to store the internal representation of the */
       /* value of this option, such as an int or XColor *.           */
       /* This field is specified as an offset in bytes from the      */
       /* start of the record. Use the Tk_Offset macro to generate    */
       /* values for it.                                              */
       /* -1 means don't store the internal representation.           */
     2 flags      fixed bin (31),
       /* Any combination of the values defined below.                */
     2 clientData type ClientData,
       /* An alternate place to put option-specific data.             */
       /* Used for the monochrome default value for colors, etc.      */
     2 typeMask   fixed bin (31);
       /* An arbitrary bit mask defined by the class manager,         */
       /* typically bits correspond to certain kinds of options such  */
       /* as all those that require a redisplay when they change.     */
       /* Tk_SetOptions returns the bit-wise OR of the typeMasks of   */
       /* all options that were changed.                              */

 /* Flag values for Tk_OptionSpec structures.  These flags are shared */
 /* by Tk_ConfigSpec structures, so be sure to coordinate any changes */
 /* carefully.                                                        */

 dcl TK_OPTION_NULL_OK          fixed bin(31) value(1);
 dcl TK_OPTION_DONT_SET_DEFAULT fixed bin(31) value(8);

 /* Macro to use to fill in "offset" fields of the Tk_OptionSpec      */
 /* struct.  Computes number of bytes from beginning of structure     */
 /* to a given field.                                                 */

 %Tk_Offset: proc (type, field) returns (char);
   dcl (type, field) char;
   return ('location (bind (: ' || type || ', sysnull() :) => ' ||
                      field || ')');
 %end Tk_Offset;
 %dcl Tk_Offset entry;

 /* The following two structures are used for error handling.  When   */
 /* configuration options are being modified, the old values are      */
 /* saved in a Tk_SavedOptions structure.  If an error occurs, then   */
 /* the contents of the structure can be used to restore all of the   */
 /* old values.  The contents of this structure are for the private   */
 /* use of Tk. No-one outside Tk should ever read or write any of     */
 /* the fields of these structures.                                   */

 define structure
   1 Tk_SavedOption,
     2 optionPtr    pointer, /* handle TkOption */
       /* Points to information that describes the option.            */
     2 valuePtr     handle Tcl_Obj,
       /* The old value of the option, in the form of a Tcl object,   */
       /* may be NULL if the value wasn't saved as an object.         */
     2 internalForm float bin(53);
       /* The old value of the option, in some internal               */
       /* representation such as an int or (XColor *).                */
       /* Valid only if optionPtr->specPtr->objOffset is < 0.         */
       /* The space must be large enough to accommodate a double, a   */
       /* long, or a pointer, right now it looks like a double is big */
       /* enough.  Also, using a double guarantees that the field is  */
       /* properly aligned for storing large values.                  */

 %if TCL_MEM_DEBUG ^= '' %then %do;
 dcl TK_NUM_SAVED_OPTIONS fixed bin(31) value(2);
 %end; %else %do;
 dcl TK_NUM_SAVED_OPTIONS fixed bin(31) value(20);
 %end;

 define structure
   1 Tk_SavedOptions,
     2 recordPtr pointer,
       /* The data structure in which to restore configuration options*/
     2 tkwin     type Tk_Window,
       /* Window associated with recordPtr,                           */
       /* needed to restore certain options.                          */
     2 numItems  fixed bin (31),
       /* The number of valid items in items field.                   */
     2 items (TK_NUM_SAVED_OPTIONS) type Tk_SavedOption,
       /* Items used to hold old values.                              */
     2 nextPtr handle Tk_SavedOptions;
       /* Points to next structure in list, needed if too many        */
       /* options changed to hold all the old values in a single      */
       /* structure.  NULL means no more structures.                  */

 /* Structure used to describe application-specific configuration     */
 /* options: indicates procedures to call to parse an option and      */
 /* to return a text string describing an option.                     */
 /* THESE ARE DEPRECATED, PLEASE USE THE NEW STRUCTURES LISTED ABOVE. */

 %if #_NO_OLD_CONFIG = '' %then %do;
 define alias Tk_OptionParseProc
        entry (type   ClientData,
               handle Tcl_Interp,
               type   Tk_Window,
               pointer, /* to char (*) varz */
               pointer, /* to a widget record */
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_OptionPrintProc
        entry (type   ClientData,
               type   Tk_Window,
               pointer, /* to a widget record */
               fixed bin (31),
               type Tcl_FreeProc byaddr)
     returns (pointer byvalue) /* to something, undocumented */
     options (byvalue nodescriptor linkage(cdecl)) limited;


 define structure
   1 Tk_CustomOption,
       2 parseProc  type Tk_OptionParseProc,
                    /* Procedure to call to parse an           */
                    /* option and store it in converted form.  */
       2 printProc  type Tk_OptionPrintProc,
                    /* Procedure to return a printable         */
                    /* string describing an existing * option. */
       2 clientData type ClientData;
                    /* Arbitrary one-word value used by option    */
                    /* parser: passed to parseProc and printProc. */
 /* Structure used to specify information for Tk_ConfigureWidget.     */
 /* Each structure gives complete information for one option,         */
 /* including how the option is specified on the command line, where  */
 /* it appears in the option database, etc.                           */

 define structure
   1 Tk_ConfigSpec,
     2 type fixed bin(31),/* Type of option, such as TK_CONFIG_COLOR. */
                          /* see definitions below. Last option in    */
                          /* table must have type TK_CONFIG_END.      */
     2 argvName pointer,  /* Switch used to specify option in argv.   */
                          /* NULL means this spec is part of a group. */
     2 dbName   pointer,  /* Name for option in option database.      */
     2 dbClass  pointer,  /* Class for option in database.            */
     2 defValue pointer,  /* Default value for option if not          */
                          /* specified in command line or database.   */
     2 offset fixed bin(31),/* Where in widget record to store value  */
                            /* use LOCATION BIF to generate values    */
                            /* for this.                              */
     2 specFlags fixed bin(31),
                          /* Any combination of the values defined    */
                          /* below, other bits are used internally    */
                          /* by tkConfig.c.                           */
     2 customPtr pointer; /* If type is TK_CONFIG_CUSTOM then this is */
                          /* a pointer to info about how to parse and */
                          /* print the option. Otherwise it is        */
                          /* irrelevant.                              */

 /* Type values for Tk_ConfigSpec structures.  See the user */
 /* documentation for details.                              */

 define ordinal Tk_ConfigTypes
   ( TK_CONFIG_BOOLEAN,
     TK_CONFIG_INT,
     TK_CONFIG_DOUBLE,
     TK_CONFIG_STRING,
     TK_CONFIG_UID,
     TK_CONFIG_COLOR,
     TK_CONFIG_FONT,
     TK_CONFIG_BITMAP,
     TK_CONFIG_BORDER,
     TK_CONFIG_RELIEF,
     TK_CONFIG_CURSOR,
     TK_CONFIG_ACTIVE_CURSOR,
     TK_CONFIG_JUSTIFY,
     TK_CONFIG_ANCHOR,
     TK_CONFIG_SYNONYM,
     TK_CONFIG_CAP_STYLE,
     TK_CONFIG_JOIN_STYLE,
     TK_CONFIG_PIXELS,
     TK_CONFIG_MM,
     TK_CONFIG_WINDOW,
     TK_CONFIG_CUSTOM,
     TK_CONFIG_END
   ) prec(31);

 /* Possible values for flags argument to Tk_ConfigureWidget: */

 dcl TK_CONFIG_ARGV_ONLY fixed bin(31) value(1);
 dcl TK_CONFIG_OBJS      fixed bin(31) value('80'xn);

 /* Possible flag values for Tk_ConfigInfo structures.  Any bits at   */
 /* or above TK_CONFIG_USER_BIT may be used by clients for selecting  */
 /* certain entries.  Before changing any values here, coordinate     */
 /* with tkConfig.c (internal-use-only flags are defined there).      */

 dcl TK_CONFIG_COLOR_ONLY       fixed bin(31) value(1);
 dcl TK_CONFIG_MONO_ONLY        fixed bin(31) value(2);
 dcl TK_CONFIG_NULL_OK          fixed bin(31) value(4);
 dcl TK_CONFIG_DONT_SET_DEFAULT fixed bin(31) value(8);
 dcl TK_CONFIG_OPTION_SPECIFIED fixed bin(15) value('10'xn);
 dcl TK_CONFIG_USER_BIT         fixed bin(15) value('100'xn);

 %end; /* #_NO_OLD_CONFIG */

 /* Structure used to specify how to handle argv options. */

 define structure
   1 Tk_ArgvInfo,
       2 key pointer, /* The key string that flags the option in the  */
                      /* argv array.                                  */
       2 type fixed bin(31),/* Indicates option type; see below.      */
       2 src pointer, /* Value to be used in setting dst, usage       */
                      /* depends on type.                             */
       2 dst pointer, /* Address of value to be modified, usage       */
                      /* depends on type.                             */
       2 help pointer;/* Documentation message describing this        */
                      /* option.                                      */


 /* Legal values for the type field of a Tk_ArgvInfo: see the user    */
 /* documentation for details.                                        */

 dcl TK_ARGV_CONSTANT          fixed bin(31) value(15);
 dcl TK_ARGV_INT               fixed bin(31) value(16);
 dcl TK_ARGV_STRING            fixed bin(31) value(17);
 dcl TK_ARGV_UID               fixed bin(31) value(18);
 dcl TK_ARGV_REST              fixed bin(31) value(19);
 dcl TK_ARGV_FLOAT             fixed bin(31) value(20);
 dcl TK_ARGV_FUNC              fixed bin(31) value(21);
 dcl TK_ARGV_GENFUNC           fixed bin(31) value(22);
 dcl TK_ARGV_HELP              fixed bin(31) value(23);
 dcl TK_ARGV_CONST_OPTION      fixed bin(31) value(24);
 dcl TK_ARGV_OPTION_VALUE      fixed bin(31) value(25);
 dcl TK_ARGV_OPTION_NAME_VALUE fixed bin(31) value(26);
 dcl TK_ARGV_END               fixed bin(31) value(27);

 /* Flag bits for passing to Tk_ParseArgv: */

 dcl TK_ARGV_NO_DEFAULTS         fixed bin(31) value('1'xn);
 dcl TK_ARGV_NO_LEFTOVERS        fixed bin(31) value('2'xn);
 dcl TK_ARGV_NO_ABBREV           fixed bin(31) value('4'xn);
 dcl TK_ARGV_DONT_SKIP_FIRST_ARG fixed bin(31) value('8'xn);

 /* Enumerated type for describing actions to be taken in response    */
 /* to a restrictProc established by Tk_RestrictEvents.               */

 define ordinal Tk_RestrictAction ( TK_DEFER_EVENT,
                                    TK_PROCESS_EVENT,
                                    TK_DISCARD_EVENT) prec(31);

 /* Priority levels to pass to Tk_AddOption: */

 dcl TK_WIDGET_DEFAULT_PRIO fixed bin(31) value(20);
 dcl TK_STARTUP_FILE_PRIO   fixed bin(31) value(40);
 dcl TK_USER_DEFAULT_PRIO   fixed bin(31) value(60);
 dcl TK_INTERACTIVE_PRIO    fixed bin(31) value(80);
 dcl TK_MAX_PRIO            fixed bin(31) value(100);

 /* Relief values returned by Tk_GetRelief: */

 dcl TK_RELIEF_FLAT   fixed bin(31) value(0);
 dcl TK_RELIEF_GROOVE fixed bin(31) value(1);
 dcl TK_RELIEF_RAISED fixed bin(31) value(2);
 dcl TK_RELIEF_RIDGE  fixed bin(31) value(3);
 dcl TK_RELIEF_SOLID  fixed bin(31) value(4);
 dcl TK_RELIEF_SUNKEN fixed bin(31) value(5);

 /* "Which" argument values for Tk_3DBorderGC: */

 dcl TK_3D_FLAT_GC  fixed bin(31) value(1);
 dcl TK_3D_LIGHT_GC fixed bin(31) value(2);
 dcl TK_3D_DARK_GC  fixed bin(31) value(3);

 /* Special EnterNotify/LeaveNotify "mode" for use in events      */
 /* generated by tkShare.c.  Pick a high enough value that it's   */
 /* unlikely to conflict with existing values (like NotifyNormal) */
 /* or any new values defined in the future.                      */

 dcl TK_NOTIFY_SHARE fixed bin(31) value(20);

 /* Enumerated type for describing a point by which to anchor */
 /* something:                                                */

 define ordinal Tk_Anchor ( TK_ANCHOR_N,
                            TK_ANCHOR_NE,
                            TK_ANCHOR_E,
                            TK_ANCHOR_SE,
                            TK_ANCHOR_S,
                            TK_ANCHOR_SW,
                            TK_ANCHOR_W,
                            TK_ANCHOR_NW,
                            TK_ANCHOR_CENTER) prec(31);

 /* Enumerated type for describing a style of justification: */

 define ordinal Tk_Justify ( TK_JUSTIFY_LEFT,
                             TK_JUSTIFY_RIGHT,
                             TK_JUSTIFY_CENTER) prec(31);

 /* The following structure is used by Tk_GetFontMetrics() to return  */
 /* information about the properties of a Tk_Font.                    */

 define structure
   1 Tk_FontMetrics,
    2 ascent    fixed bin(31),
                /* The amount in pixels that the tallest letter       */
                /* sticks up above the baseline, plus any extra       */
                /* blank space added by the designer of the font.     */
    2 descent   fixed bin(31),
                /* The largest amount in pixels that any letter       */
                /* sticks below the baseline, plus any extra blank    */
                /* space added by the designer of the font.           */
    2 linespace fixed bin(31);
                /* The sum of the ascent and descent. How far apart   */
                /* two lines of text in the same font should be       */
                /* placed so that none of the characters in one line  */
                /* overlap any of the characters in the other line.   */


 /* Flags passed to Tk_MeasureChars: */

 dcl TK_WHOLE_WORDS  fixed bin(31) value(1);
 dcl TK_AT_LEAST_ONE fixed bin(31) value(2);
 dcl TK_PARTIAL_OK   fixed bin(31) value(4);

 /* Flags passed to Tk_ComputeTextLayout: */

 dcl TK_IGNORE_TABS     fixed bin(31) value(8);
 dcl TK_IGNORE_NEWLINES fixed bin(31) value(16);

 /* Each geometry manager (the packer, the placer, etc.) is           */
 /* represented by a structure of the following form, which indicates */
 /* procedures to invoke in the geometry manager to carry out certain */
 /* functions.                                                        */

 define alias Tk_GeomRequestProc
        entry (type   ClientData,
               type   Tk_Window)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_GeomLostSlaveProc
        entry (type   ClientData,
               type   Tk_Window)
        options (byvalue nodescriptor linkage(cdecl)) limited;


 define structure
   1 Tk_GeomMgr,
    2 name pointer,       /* Name of the geometry manager (command */
                          /* used to invoke it, or name of widget  */
                          /* class that allows embedded widgets).  */
    2 requestProc  type Tk_GeomRequestProc,
                          /* Procedure to invoke when a slave's      */
                          /* requested geometry changes.             */
    2 lostSlaveProc type Tk_GeomLostSlaveProc;
                          /* Procedure to invoke when a slave is     */
                          /* taken away from one geometry manager    */
                          /* by another. NULL means geometry manager */
                          /* doesn't care when slaves are lost.      */

 /* Result values returned by Tk_GetScrollInfo: */

 dcl TK_SCROLL_MOVETO fixed bin(31) value(1);
 dcl TK_SCROLL_PAGES  fixed bin(31) value(2);
 dcl TK_SCROLL_UNITS  fixed bin(31) value(3);
 dcl TK_SCROLL_ERROR  fixed bin(31) value(4);

 /*------------------------------------------------------------------ */
 /*                                                                   */
 /* Extensions to the X event set                                     */
 /*                                                                   */
 /*------------------------------------------------------------------ */

 dcl VirtualEvent     fixed bin (31) value (LASTEvent);
 dcl ActivateNotify   fixed bin (31) value (LASTEvent + 1);
 dcl DeactivateNotify fixed bin (31) value (LASTEvent + 2);
 dcl MouseWheelEvent  fixed bin (31) value (LASTEvent + 3);
 dcl TK_LASTEVENT     fixed bin (31) value (LASTEvent + 4);

 dcl MouseWheelMask   fixed bin (31) value (raise2 (1, 28));
 dcl ActivateMask     fixed bin (31) value (raise2 (1, 29));
 dcl VirtualEventMask fixed bin (31) value (raise2 (1, 30));


 /* A virtual event shares most of its fields with the XKeyEvent and  */
 /* XButtonEvent structures.  99% of the time a virtual event will be */
 /* an abstraction of a key or button event, so this structure        */
 /* provides the most information to the user.  The only difference   */
 /* is the changing of the detail field for a virtual event so that   */
 /* it holds the name of the virtual event being triggered.           */

 define structure
   1 XVirtualEvent,
    2 type    fixed bin(31),
    2 serial  fixed bin(32) unsigned,
             /* # of last request processed by server */
    2 send_event fixed bin (31),
             /* True if this came from a SendEvent request */
    2 display pointer,/* Display the event was read from */
    2 event   type Window,/* Window on which event was requested. */
    2 root    type Window,/* root window that the event occured on */
    2 subwindow type Window,/* child window */
    2 time    type Time,/* milliseconds */
    2 x       fixed bin(31), /* pointer x, y coordinates  */
    2 y       fixed bin(31), /* .. in event window */
    2 x_root  fixed bin(31),/* coordinates relative to root */
    2 y_root  fixed bin(31),/* coordinates relative to root */
    2 state   fixed bin(32) unsigned,/* key or button mask */
    2 name    type Tk_Uid,/* Name of virtual event. */
    2 same_screen fixed bin (31);/* same screen flag */

 define structure
   1 XActivateDeactivateEvent,
    2 type    fixed bin(31),
    2 serial  fixed bin(32) unsigned,
             /* # of last request processed by server */
    2 send_event fixed bin (31),
             /* True if this came from a SendEvent request */
    2 display pointer,    /* Display the event was read from */
    2 window  type Window;/* Window in which event occurred. */

 define alias XActivateEvent   type XActivateDeactivateEvent;
 define alias XDeactivateEvent type XActivateDeactivateEvent;

 /*------------------------------------------------------------------ */
 /*                                                                   */
 /* Macros for querying Tk_Window structures.  See the                */
 /* manual entries for documentation.                                 */
 /*                                                                   */
 /*------------------------------------------------------------------ */


 /* The structure below is needed by the macros below so that they    */
 /* can access the fields of a Tk_Window. The fields not needed by    */
 /* the macros are declared as "dummyX". The structure has its own    */
 /* type in order to prevent applications from accessing Tk_Window    */
 /* fields except using official macros.  WARNING!! The structure     */
 /* definition must be kept consistent with the TkWindow structure in */
 /* tkInt.h. If you change one, then change the other.  See the       */
 /* declaration in tkInt.h for documentation on what the fields are   */
 /* used for internally.                                              */

 define structure
   1 Tk_FakeWin,
    2 display   handle Display,
    2 dummy1    pointer,
    2 screenNum fixed bin(31),
    2 visual    handle Visual,
    2 depth     fixed bin(31),
    2 window    type Window,
    2 dummy2    pointer,
    2 dummy3    pointer,
    2 parentPtr type Tk_Window,
    2 dummy4    pointer,
    2 dummy5    pointer,
    2 pathName  pointer,
    2 nameUid   type Tk_Uid,
    2 classUid  type Tk_Uid,
    2 changes   type XWindowChanges,
    2 dummy6    fixed bin(32) unsigned,
    2 atts      type XSetWindowAttributes,
    2 dummy7    fixed bin(32) unsigned,
    2 flags     fixed bin(32) unsigned,
    2 dummy8    pointer,
 %dcl TK_USE_INPUT_METHODS char ext;
 %if TK_USE_INPUT_METHODS = 'Y' %then %do;
    2 dummy9    type XIC,
 %end; /* TK_USE_INPUT_METHODS */
    2 dummy10   pointer, /* to ClientData */
    2 dummy11   fixed bin(31),
    2 dummy12   fixed bin(31),
    2 dummy13   pointer,
    2 dummy14   pointer,
    2 dummy15   type ClientData,
    2 reqWidth  fixed bin(31),
    2 reqHeight fixed bin(31),
    2 internalBorderWidth fixed bin(31),
    2 dummy16   pointer,
    2 dummy17   pointer,
    2 dummy18   type ClientData,
    2 dummy19   pointer;

 %Tk_Display: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => display)');
 %end Tk_Display;
 %dcl Tk_Display entry;

 %Tk_ScreenNumber: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => screenNum)');
 %end Tk_ScreenNumber;
 %dcl Tk_ScreenNumber entry;

 %Tk_Screen: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('ScreenOfDisplay (Tk_Display (' || tkwin || '), ' ||
                            'Tk_ScreenNumber (' || tkwin || ')) ');
 %end Tk_Screen;
 %dcl Tk_Screen entry;

 %Tk_Depth: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => depth)');
 %end Tk_Depth;
 %dcl Tk_Depth entry;

 %Tk_Visual: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => visual)');
 %end Tk_Visual;
 %dcl Tk_Visual entry;

 %Tk_WindowId: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => window)');
 %end Tk_WindowId;
 %dcl Tk_WindowId entry;

 %Tk_PathName: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => pathName)');
 %end Tk_PathName;
 %dcl Tk_PathName entry;

 %Tk_Name: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => nameUid)');
 %end Tk_Name;
 %dcl Tk_Name entry;

 %Tk_Class: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => classUid)');
 %end Tk_Class;
 %dcl Tk_Class entry;

 %Tk_X: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => changes.x)');
 %end Tk_X;
 %dcl Tk_X entry;

 %Tk_Y: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => changes.y)');
 %end Tk_Y;
 %dcl Tk_Y entry;

 %Tk_Width: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => changes.width)');
 %end Tk_Width;
 %dcl Tk_Width entry;

 %Tk_Height: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => changes.height)');
 %end Tk_Height;
 %dcl Tk_Height entry;

 %Tk_Changes: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('handle ((' || tkwin || ') => changes)');
 %end Tk_Changes;
 %dcl Tk_Changes entry;

 %Tk_Attributes: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('handle ((' || tkwin || ') => atts)');
 %end Tk_Attributes;
 %dcl Tk_Attributes entry;

 /* Note: the C Tk_IsXxx macros return an integer, we return a bit(1) */
 %Tk_IsEmbedded: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('(iand ((' || tkwin || ') => flags, TK_EMBEDDED) ^= 0)');
 %end Tk_IsEmbedded;
 %dcl Tk_IsEmbedded entry;

 %Tk_IsContainer: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('(iand ((' || tkwin || ') => flags, TK_CONTAINER) ^= 0)');
 %end Tk_IsContainer;
 %dcl Tk_IsContainer entry;

 %Tk_IsMapped: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('(iand ((' || tkwin || ') => flags, TK_MAPPED) ^= 0)');
 %end Tk_IsMapped;
 %dcl Tk_IsMapped entry;

 %Tk_IsTopLevel: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('(iand ((' || tkwin || ') => flags, TK_TOP_LEVEL) ^= 0)');
 %end Tk_IsTopLevel;
 %dcl Tk_IsTopLevel entry;

 %Tk_ReqWidth: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => reqWidth)');
 %end Tk_ReqWidth;
 %dcl Tk_ReqWidth entry;

 %Tk_ReqHeight: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => reqHeight)');
 %end Tk_ReqHeight;
 %dcl Tk_ReqHeight entry;

 %Tk_InternalBorderWidth: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => internalBorderWidth)');
 %end Tk_InternalBorderWidth;
 %dcl Tk_InternalBorderWidth entry;

 %Tk_Parent: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => parentPtr)');
 %end Tk_Parent;
 %dcl Tk_Parent entry;

 %Tk_Colormap: proc (tkwin) returns (char);
   dcl tkwin char;
   return ('((' || tkwin || ') => atts.colormap)');
 %end Tk_Colormap;
 %dcl Tk_Colormap entry;

 /* Flag values for TkWindow (and Tk_FakeWin) structures are:         */
 /*                                                                   */
 /* TK_MAPPED:        1 means window is currently mapped,             */
 /*                   0 means unmapped.                               */
 /* TK_TOP_LEVEL:     1 means this is a top-level window (it was or   */
 /*                   will be created as a child of a root window).   */
 /* TK_ALREADY_DEAD:  1 means the window is in the process of         */
 /*                   being destroyed already.                        */
 /* TK_NEED_CONFIG_NOTIFY:  1 means that the window has been          */
 /*                   reconfigured before it was made to exist.       */
 /*                   At the time of making it exist a ConfigureNotify*/
 /*                   event needs to be generated.                    */
 /* TK_GRAB_FLAG:     Used to manage grabs.  See tkGrab.c for         */
 /*                   details.                                        */
 /* TK_CHECKED_IC:    1 means we've already tried to get an input     */
 /*                   context for this window;  if the ic field       */
 /*                   is NULL it means that there isn't a context     */
 /*                   for the field.                                  */
 /* TK_DONT_DESTROY_WINDOW: 1 means that Tk_DestroyWindow should not  */
 /*                   invoke XDestroyWindow to destroy this widget's  */
 /*                   X window.  The flag is set when the window      */
 /*                   has already been destroyed elsewhere (e.g.      */
 /*                   by another application) or when it will be      */
 /*                   destroyed later (e.g. by destroying its parent).*/
 /* TK_WM_COLORMAP_WINDOW:  1 means that this window has at some time */
 /*                   appeared in the WM_COLORMAP_WINDOWS property    */
 /*                   for its toplevel, so we have to remove it       */
 /*                   from that property if the window is             */
 /*                   deleted and the toplevel isn't.                 */
 /* TK_EMBEDDED:      1 means that this window (which must be a       */
 /*                   toplevel) is not a free-standing window but     */
 /*                   rather is embedded in some other application.   */
 /* TK_CONTAINER:     1 means that this window is a container, and    */
 /*                   that some other application (either in          */
 /*                   this process or elsewhere) may be               */
 /*                   embedding itself inside the window.             */
 /* TK_BOTH_HALVES:   1 means that this window is used for            */
 /*                   application embedding (either as                */
 /*                   container or embedded application), and         */
 /*                   both the containing and embedded halves         */
 /*                   are associated with windows in this             */
 /*                   particular process.                             */
 /* TK_DEFER_MODAL:   1 means that this window has deferred a modal   */
 /*                   loop until all of the bindings for the current  */
 /*                   event have been invoked.                        */
 /* TK_WRAPPER:       1 means that this window is the extra           */
 /*                   wrapper window created around a toplevel        */
 /*                   to hold the menubar under Unix.  See            */
 /*                   tkUnixWm.c for more information.                */
 /* TK_REPARENTED:    1 means that this window has been reparented    */
 /*                   so that as far as the window system is          */
 /*                   concerned it isn't a child of its Tk            */
 /*                   parent.  Initially this is used only for        */
 /*                   special Unix menubar windows.                   */


 dcl TK_MAPPED              fixed bin(31) value(1);
 dcl TK_TOP_LEVEL           fixed bin(31) value(2);
 dcl TK_ALREADY_DEAD        fixed bin(31) value(4);
 dcl TK_NEED_CONFIG_NOTIFY  fixed bin(31) value(8);
 dcl TK_GRAB_FLAG           fixed bin(31) value('10'xn);
 dcl TK_CHECKED_IC          fixed bin(31) value('20'xn);
 dcl TK_DONT_DESTROY_WINDOW fixed bin(31) value('40'xn);
 dcl TK_WM_COLORMAP_WINDOW  fixed bin(31) value('80'xn);
 dcl TK_EMBEDDED            fixed bin(31) value('100'xn);
 dcl TK_CONTAINER           fixed bin(31) value('200'xn);
 dcl TK_BOTH_HALVES         fixed bin(31) value('400'xn);
 dcl TK_DEFER_MODAL         fixed bin(31) value('800'xn);
 dcl TK_WRAPPER             fixed bin(31) value('1000'xn);
 dcl TK_REPARENTED          fixed bin(31) value('2000'xn);

 /*------------------------------------------------------------------ */
 /*                                                                   */
 /* Procedure prototypes and structures used for defining new canvas  */
 /* items:                                                            */
 /*                                                                   */
 /*------------------------------------------------------------------ */

 define ordinal Tk_State
   ( TK_STATE_NULL      value(-1),
     TK_STATE_ACTIVE,
     TK_STATE_DISABLED,
     TK_STATE_NORMAL,
     TK_STATE_HIDDEN
   ) prec(31);

 define structure
   1 Tk_SmoothMethod,
     2 name      pointer, /* to char (*) varz */
     2 coordProc entry (type Tk_Canvas,
                        float bin (53) byaddr,
                        fixed bin (31),
                        fixed bin (31),
                        (*) type XPoint byaddr,
                        (*) float bin (53) byaddr)
                 returns (fixed bin (31) byvalue optional)
                 options (byvalue nodescriptor linkage(cdecl)) limited,
     2 postscriptProc
                 entry (handle Tcl_Interp,
                        type Tk_Canvas,
                        float bin (53) byaddr,
                        fixed bin (31),
                        fixed bin (31))
                 options (byvalue nodescriptor linkage(cdecl)) limited;

 /* For each item in a canvas widget there exists one record with     */
 /* the following structure.  Each actual item is represented by      */
 /* a record with the following stuff at its beginning, plus          */
 /* additional type-specific stuff after that.                        */

 dcl TK_TAG_SPACE fixed bin(31) value(3);

 define structure
   1 Tk_Item,
    2 id fixed bin(31),      /* Unique identifier for this item       */
                             /* (also serves as first tag for item).  */
    2 nextPtr handle Tk_Item,/* Next in display list of all items in  */
                             /* this canvas. Later items in list are  */
                             /* drawn on top of earlier ones.         */
    2 staticTagSpace (TK_TAG_SPACE) type Tk_Uid,/* Built-in space     */
                             /* for limited # of tags.                */
    2 tagPtr  pointer,       /* Pointer to array of tags.     */
                             /* Usually points to staticTagSpace, but */
                             /* may point to malloc-ed space if there */
                             /* are lots of tags.                     */
    2 tagSpace fixed bin(31),/* Total amount of tag space available   */
                             /* at tagPtr.                            */
    2 numTags fixed bin(31), /* Number of tag slots actually used at  */
                             /* tagPtr.                               */
    2 typePtr handle Tk_ItemType,/* Table of procedures that          */
                             /* implement this type of item.          */
    2 x1      fixed bin(31), /* Bounding box for item, in integer     */
    2 y1      fixed bin(31), /* canvas units. Set by item-specific    */
    2 x2      fixed bin(31), /* code and guaranteed to contain every  */
    2 y2      fixed bin(31), /* pixel drawn in item. Item area        */
                             /* includes x1 and y1 but not x2 and y2. */
    2 prevPtr handle Tk_Item,/* Previous in display list of all items */
                             /* in this canvas. Later items in list   */
                             /* are drawn just below earlier ones.    */
    2 state   type Tk_State, /* state of item                         */
    2 reserved1    pointer,  /* reserved for future use               */
    2 redraw_flags fixed bin (31);   /* some flags used in the canvas */
    /*----------------------------------------------------------------*/
    /* Starting here is additional type-specific stuff;  see the      */
    /* declarations for individual types to see what is part of       */
    /* each type.  The actual space below is determined by the        */
    /* "itemInfoSize" of the type's Tk_ItemType record.               */
    /*----------------------------------------------------------------*/


 /* Flag bits for canvases (redraw_flags):                            */
 /*                                                                   */
 /* TK_ITEM_STATE_DEPENDANT - 1 means that object needs to be         */
 /*	                        redrawn if the canvas state changes.      */
 /* TK_ITEM_DONT_REDRAW -   1 means that the object redraw is already */
 /*                         been prepared, so the general canvas code */
 /*                         doesn't need to do that any more.         */

 dcl TK_ITEM_STATE_DEPENDANT fixed bin (31) value (1);
 dcl TK_ITEM_DONT_REDRAW     fixed bin (31) value (2);

 /* Records of the following type are used to describe a type of      */
 /* item (e.g. lines, circles, etc.) that can form part of a          */
 /* canvas widget.                                                    */

 %dcl USE_OLD_CANVAS char;
 %if USE_OLD_CANVAS ^= '' %then %do;
 define alias Tk_ItemCreateProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer) /* to (*) pointer to char (*) varz */
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemConfigureProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer, /* to (*) pointer to char (*) varz */
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemCoordProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer) /* to (*) pointer to char (*) varz */
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end; %else %do;
 define alias Tk_ItemCreateProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer) /* to (*) handle Tcl_Obj byaddr nonasgn */
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemConfigureProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer, /* to (*) handle Tcl_Obj byaddr nonasgn */
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemCoordProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer) /* to (*) handle Tcl_Obj byaddr nonasgn */
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end;

 define alias Tk_ItemDeleteProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               handle Display)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemDisplayProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               handle Display,
               type   Drawable,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemPointProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               float bin (53) byaddr)
     returns (float bin (53) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemAreaProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               float bin (53) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemPostscriptProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemScaleProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               float bin (53) byaddr,
               float bin (53) byaddr,
               float bin (53) byaddr,
               float bin (53) byaddr)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemTranslateProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               float bin (53) byaddr,
               float bin (53) byaddr)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemIndexProc
        entry (handle Tcl_Interp,
               type   Tk_Canvas,
               handle Tk_Item,
               pointer, /* to char (*) varz */
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemCursorProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31))
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemSelectionProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               pointer, /* to char (*) varz */
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemInsertProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               char (*) varz byaddr)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ItemDCharsProc
        entry (type   Tk_Canvas,
               handle Tk_Item,
               fixed bin (31),
               fixed bin (31))
     options (byvalue nodescriptor linkage(cdecl)) limited;


 %if #_NO_OLD_CONFIG = '' %then %do;

 define structure
   1 Tk_ItemType,
     2 name           pointer, /* to char (*) varz */
          /* The name of this type of item, such as "line". */
     2 itemSize       fixed bin(31),
          /* Total amount of space needed for item's record. */
     2 createProc     type Tk_ItemCreateProc,
          /* Procedure to create a new item of this type. */
     2 configSpecs    handle Tk_ConfigSpec,
          /* Pointer to array of configuration specs for this type. */
          /* Used for returning configuration info. */
     2 configProc     type Tk_ItemConfigureProc,
          /* Procedure to call to change configuration options. */
     2 coordProc      type Tk_ItemCoordProc,
          /* Procedure to call to get and set the item's coordinates. */
     2 deleteProc     type Tk_ItemDeleteProc,
          /* Procedure to delete existing item of this type. */
     2 displayProc    type Tk_ItemDisplayProc,
          /* Procedure to display items of this type. */
     2 alwaysRedraw   fixed bin(31),
          /* Non-zero means displayProc should  be called even when */
          /* the item has been moved off-screen. */
     2 pointProc      type Tk_ItemPointProc,
          /* Computes distance from item to a given point. */
     2 areaProc       type Tk_ItemAreaProc,
          /* Computes whether item is inside, outside, or */
          /* overlapping an area.                         */
     2 postscriptProc type Tk_ItemPostscriptProc,
          /* Procedure to write a Postscript description for items */
          /* of this type. */
     2 scaleProc      type Tk_ItemScaleProc,
          /* Procedure to rescale items of this type. */
     2 translateProc  type Tk_ItemTranslateProc,
          /* Procedure to translate items of this type. */
     2 indexProc      type Tk_ItemIndexProc,
          /* Procedure to determine index of indicated character. */
          /* NULL if item doesn't support indexing. */
     2 icursorProc    type Tk_ItemCursorProc,
          /* Procedure to set insert cursor pos. to just before */
          /* a given position. */
     2 selectionProc  type Tk_ItemSelectionProc,
          /* Procedure to return selection (in STRING format) */
          /* when it is in this item. */
     2 insertProc     type Tk_ItemInsertProc,
          /* Procedure to insert something into an item. */
     2 dCharsProc     type Tk_ItemDCharsProc,
          /* Procedure to delete characters from an item. */
     2 nextPtr        handle Tk_ItemType,
          /* Used to link types together into a list. */
     2 reserved1 pointer, /* Reserved for future extension. */
     2 reserved2 fixed bin(31),	/* Carefully compatible with */
     2 reserved3 pointer,       /* Jan Nijtmans dash patch */
     2 reserved4 pointer;
 %end;

 /* The following structure provides information about the selection  */
 /* and the insertion cursor.  It is needed by only a few items, such */
 /* as those that display text.  It is shared by the generic canvas   */
 /* code and the item-specific code, but most of the fields should be */
 /* written only by the canvas generic code.                          */

 define structure
   1 Tk_CanvasTextInfo,
     2 selBorder         type Tk_3DBorder,
          /* Border and background for selected characters. */
          /* Read-only to items.*/
     2 selBorderWidth    fixed bin(31),
          /* Width of border around selection. Read-only to items. */
     2 selFgColorPtr     handle XColor,
          /* Foreground color for selected text. Read-only to items. */
     2 selItemPtr        handle Tk_Item,
          /* Pointer to selected item. NULL means selection isn't in */
          /* this canvas. Writable by items. */
     2 selectFirst       fixed bin(31),
          /* Index of first selected character. Writable by items. */
     2 selectLast        fixed bin(31),
          /* Index of last selected character. Writable by items. */
     2 anchorItemPtr     handle Tk_Item,
          /* Item corresponding to "selectAnchor": not necessarily */
          /* selItemPtr. Read-only to items. */
     2 selectAnchor      fixed bin(31),
          /* Fixed end of selection (i.e. "select to" operation will */
          /* use this as one end of the selection). Writable by items.*/
     2 insertBorder      type Tk_3DBorder,
          /* Used to draw vertical bar for insertion cursor. */
          /* Read-only to items. */
     2 insertWidth       fixed bin(31),
          /* Total width of insertion cursor. Read-only to items. */
     2 insertBorderWidth fixed bin(31),
          /* Width of 3-D border around insert cursor. */
          /* Read-only to items. */
     2 focusItemPtr      handle Tk_Item,
          /* Item that currently has the input focus, or NULL */
          /* if no such item. Read-only to items. */
     2 gotFocus          fixed bin(31),
          /* Non-zero means that the canvas widget has the input */
          /* focus. Read-only to items.*/
     2 cursorOn          fixed bin(31);
          /* Non-zero means that an insertion cursor should be */
          /* displayed in focusItemPtr. Read-only to items.    */

 /* Structures used for Dashing and Outline. */

 define structure
   1 Tk_Dash,
     2 number  fixed bin (31),
     2 pattern union,
       3 pt    pointer,
       3 array char (4); /* 4 = size (: pointer :) */


 define structure
   1 Tk_TSOffset,
     2 flags   fixed bin (31), /* flags, see below for possible values*/
     2 xoffset fixed bin (31), /* x offset */
     2 yoffset fixed bin (31); /* y offset */

 /* Bit fields in Tk_Offset.flags: */

 dcl TK_OFFSET_INDEX    fixed bin (31) value (1);
 dcl TK_OFFSET_RELATIVE fixed bin (31) value (2);
 dcl TK_OFFSET_LEFT     fixed bin (31) value (4);
 dcl TK_OFFSET_CENTER   fixed bin (31) value (8);
 dcl TK_OFFSET_RIGHT    fixed bin (31) value (16);
 dcl TK_OFFSET_TOP      fixed bin (31) value (32);
 dcl TK_OFFSET_MIDDLE   fixed bin (31) value (64);
 dcl TK_OFFSET_BOTTOM   fixed bin (31) value (128);

 define structure
   1 Tk_Outline,
     2 gc            type GC,        /* Graphics context. */
     2 width         float bin (53), /* Width of outline. */
     2 activeWidth   float bin (53), /* Width of outline. */
     2 disabledWidth float bin (53), /* Width of outline. */
     2 offset        fixed bin (31), /* Dash offset */
     2 dash          type Tk_Dash,/* Dash pattern */
     2 activeDash    type Tk_Dash,/* Dash pattern if state is active */
     2 disabledDash  type Tk_Dash,/* Dash pattern if state is disabled*/
     2 reserved1     pointer,     /* reserved for future expansion */
     2 reserved2     pointer,
     2 reserved3     pointer,
     2 tsoffset      type Tk_TSOffset, /* stipple offset for outline */
     2 color         handle XColor, /* Outline color. */
     2 activeColor   handle XColor,
                      /* Outline color if state is active. */
     2 disabledColor  handle XColor,
                      /* Outline color if state is disabled. */
     2 stipple        type Pixmap, /* Outline Stipple pattern. */
     2 activeStipple  type Pixmap,
                      /* Outline Stipple pattern if state is active.  */
     2 disabledStipple type Pixmap;
                      /* Outline Stipple pattern if state is disabled.*/

 /*------------------------------------------------------------------ */
 /*                                                                   */
 /* Procedure prototypes and structures used for managing images:     */
 /*                                                                   */
 /*------------------------------------------------------------------ */

 %dcl USE_OLD_IMAGE char;
 %if USE_OLD_IMAGE ^= '' %then %do;
 define alias Tk_ImageCreateProc
        entry (handle Tcl_Interp,
               pointer, /* to char (*) varz */
               fixed bin (31),
               pointer, /* to (*) pointer to char (*) varz */
               handle Tk_ImageType,
               type Tk_ImageMaster,
               type ClientData byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end; %else %do;
 define alias Tk_ImageCreateProc
        entry (handle Tcl_Interp,
               pointer, /* to char (*) varz */
               fixed bin (31),
               pointer, /* to (*) handle Tcl_Obj byaddr nonasgn */
               handle Tk_ImageType,
               type Tk_ImageMaster,
               type ClientData byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end;

 define alias Tk_ImageGetProc
        entry (type Tk_Window,
               type ClientData)
     returns (type ClientData byvalue)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageDisplayProc
        entry (type ClientData,
               handle Display,
               type Drawable,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageFreeProc
        entry (type ClientData,
               handle Display)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageDeleteProc
        entry (type ClientData)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageChangedProc
        entry (type ClientData,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     options (byvalue nodescriptor linkage(cdecl)) limited;

 /* The following structure represents a particular type of image     */
 /* (bitmap, xpm image, etc.). It provides information common to  all */
 /* images of that type, such as the type name and a collection of    */
 /* procedures in the image manager that respond to various events.   */
 /* Each image manager is represented by one of these structures.     */


 define structure
   1 Tk_ImageType,
    2 name        pointer, /* Name of image type.                     */
    2 createProc  type Tk_ImageCreateProc,
          /* Procedure to call to create a new image of this type.    */
    2 getProc     type Tk_ImageGetProc,
          /* Procedure to call the first time Tk_GetImageis called in */
          /* a new way (new visual or screen). */
    2 displayProc type Tk_ImageDisplayProc,
          /* Call to draw image, in response to Tk_RedrawImage calls. */
    2 freeProc    type Tk_ImageFreeProc,
          /* Procedure to call whenever Tk_FreeImage is called to     */
          /* release an instance of an image. */
    2 deleteProc  type Tk_ImageDeleteProc,
          /* Procedure to call to delete image. It will not be called */
          /* until after freeProc has been called for each instance   */
          /* of the image.                                            */
    2 nextPtr     handle Tk_ImageType,
         /* Next in list of all image types currently known.          */
         /* Filled in by Tk, not by image manager.                    */
    2 reserved1   pointer; /* Reserved for future extension.          */


 /* ----------------------------------------------------------------- */
 /*                                                                   */
 /* Additional definitions used to manage images of type "photo".     */
 /*                                                                   */
 /* ----------------------------------------------------------------- */


 /* The following type is used to identify a particular photo image   */
 /* to be manipulated:                                                */

 define alias Tk_PhotoHandle pointer;

 /* The following structure describes a block of pixels in memory:    */

 define structure
   1 Tk_PhotoImageBlock,
    2 pixelPtr  pointer,       /* Pointer to the first pixel.         */
    2 width     fixed bin(31), /* Width of block, in pixels.          */
    2 height    fixed bin(31), /* Height of block, in pixels.         */
    2 pitch     fixed bin(31), /* Address difference between corres-  */
                               /* ponding pixels in successive lines. */
    2 pixelSize fixed bin(31), /* Address difference between          */
                               /* successive pixels in the same line. */
    2 offset(3) fixed bin(31), /* Address differences between the     */
                               /* red, green and blue components of   */
                               /* the pixel and the pixel as a whole. */
    2 reserved  fixed bin(31); /* Reserved for extensions (dash patch)*/

 /* Procedure prototypes and structures used in reading and           */
 /* writing photo images:                                             */

 %if USE_OLD_IMAGE ^= '' %then %do;
 define alias Tk_ImageFileMatchProc
        entry (type Tcl_Channel,
               pointer, /* to char (*) varz */
               pointer, /* to char (*) varz */
               fixed bin (31) byaddr,
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringMatchProc
        entry (pointer, /* to char (*) varz */
               pointer, /* to char (*) varz */
               fixed bin (31) byaddr,
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageFileReadProc
        entry (handle Tcl_Interp,
               type Tcl_Channel,
               pointer, /* to char (*) varz */
               pointer, /* to char (*) varz */
               type Tk_PhotoHandle,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringReadProc
        entry (handle Tcl_Interp,
               pointer, /* to char (*) varz */
               pointer, /* to char (*) varz */
               type Tk_PhotoHandle,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageFileWriteProc
        entry (handle Tcl_Interp,
               pointer, /* to char (*) varz */
               pointer, /* to char (*) varz */
               type Tk_PhotoImageBlock byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringWriteProc
        entry (handle Tcl_Interp,
               type Tcl_Dstring byaddr,
               pointer, /* to char (*) varz */
               type Tk_PhotoImageBlock byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end; %else %do;
 define alias Tk_ImageFileMatchProc
        entry (type Tcl_Channel,
               pointer, /* to char (*) varz nonasgn */
               handle Tcl_Obj,
               fixed bin (31) byaddr,
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringMatchProc
        entry (handle Tcl_Obj,
               handle Tcl_Obj,
               fixed bin (31) byaddr,
               fixed bin (31) byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageFileReadProc
        entry (handle Tcl_Interp,
               type Tcl_Channel,
               pointer, /* to char (*) varz nonasgn */
               handle Tcl_Obj,
               type Tk_PhotoHandle,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringReadProc
        entry (handle Tcl_Interp,
               handle Tcl_Obj,
               handle Tcl_Obj,
               type Tk_PhotoHandle,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31),
               fixed bin (31))
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageFileWriteProc
        entry (handle Tcl_Interp,
               pointer, /* to char (*) varz nonasgn */
               handle Tcl_Obj,
               type Tk_PhotoImageBlock byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ImageStringWriteProc
        entry (handle Tcl_Interp,
               type Tcl_Dstring byaddr,
               handle Tcl_Obj,
               type Tk_PhotoImageBlock byaddr)
     returns (fixed bin (31) byvalue optional)
     options (byvalue nodescriptor linkage(cdecl)) limited;
 %end;

 /* The following structure represents a particular file format for   */
 /* storing images (e.g., PPM, GIF, JPEG, etc.).  It provides         */
 /* information to allow image files of that format to be recognized  */
 /* and read into a photo image.                                      */

 define structure
   1 Tk_PhotoImageFormat,
    2 name            pointer, /* Name of image file format           */
    2 fileMatchProc   type Tk_ImageFileMatchProc,
          /* Procedure to call to determine whether                   */
          /* an image file matches this format.                       */
    2 stringMatchProc type Tk_ImageStringMatchProc,
         /* Procedure to call to determine whether                    */
         /* the data in a string matches this format.                 */
    2 fileReadProc    type Tk_ImageFileReadProc,
         /* Procedure to call to read data from                       */
         /* an image file into a photo image.                         */
    2 stringReadProc  type Tk_ImageStringReadProc,
         /* Procedure to call to read data from                       */
         /* a string into a photo image.                              */
    2 fileWriteProc   type Tk_ImageFileWriteProc,
         /* Procedure to call to write data from                      */
         /* a photo image to a file.                                  */
    2 stringWriteProc type Tk_ImageStringWriteProc,
          /* Procedure to call to obtain a string representation of   */
          /* the data in a photo image.                               */
    2 nextPtr         handle Tk_PhotoImageFormat;
          /* Next in list of all photo image formats currently known. */
          /* Filled in by Tk, not by image format handler.            */


 dcl Tk_CreateOldImageType
     entry (type Tk_ImageType byaddr)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tk_CreateOldImageType');

 dcl Tk_CreateOldPhotoImageFormat
     entry (type Tk_PhotoImageFormat byaddr)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tk_CreateOldPhotoImageFormat');

 %if USE_TK_STUBS = '' & USE_OLD_IMAGE ^= '' %then %do;
 %dcl Tk_CreateImageType char;
 %Tk_CreateImageType = 'Tk_CreateOldImageType';
 dcl Tk_CreatePhotoImageFormat char;
 %Tk_CreatePhotoImageFormat = 'Tk_CreateOldPhotoImageFormat';
 %end;


 /* ----------------------------------------------------------------- */
 /*                                                                   */
 /* The definitions below provide backward compatibility for          */
 /* functions and types related to event handling that used to        */
 /* be in Tk but have moved to Tcl.                                   */
 /*                                                                   */
 /* ----------------------------------------------------------------- */

 dcl TK_READABLE      fixed bin(31) value (TCL_READABLE);
 dcl TK_WRITABLE      fixed bin(31) value (TCL_WRITABLE);
 dcl TK_EXCEPTION     fixed bin(31) value (TCL_EXCEPTION);

 dcl TK_DONT_WAIT     fixed bin(31) value (TCL_DONT_WAIT);
 dcl TK_X_EVENTS      fixed bin(31) value (TCL_WINDOW_EVENTS);
 dcl TK_WINDOW_EVENTS fixed bin(31) value (TCL_WINDOW_EVENTS);
 dcl TK_FILE_EVENTS   fixed bin(31) value (TCL_FILE_EVENTS);
 dcl TK_TIMER_EVENTS  fixed bin(31) value (TCL_TIMER_EVENTS);
 dcl TK_IDLE_EVENTS   fixed bin(31) value (TCL_IDLE_EVENTS);
 dcl TK_ALL_EVENTS    fixed bin(31) value (TCL_ALL_EVENTS);

 %dcl Tk_IdleProc     char ext; %Tk_IdleProc     = 'Tcl_IdleProc';
 %dcl Tk_FileProc     char ext; %Tk_FileProc     = 'Tcl_FileProc';
 %dcl Tk_TimerProc    char ext; %Tk_TimerProc    = 'Tcl_TimerProc';
 %dcl Tk_TimerToken   char ext; %Tk_TimerToken   = 'Tcl_TimerToken';

 %dcl Tk_BackgroundError char ext;
 %Tk_BackgroundError='Tcl_BackgroundError';
 %dcl Tk_CancelIdleCall char ext;
 %Tk_CancelIdleCall='Tcl_CancelIdleCall';
 %dcl Tk_CreateFileHandler char ext;
 %Tk_CreateFileHandler='Tcl_CreateFileHandler';
 %dcl Tk_CreateTimerHandler char ext;
 %Tk_CreateTimerHandler='Tcl_CreateTimerHandler';
 %dcl Tk_DeleteFileHandler char ext;
 %Tk_DeleteFileHandler='Tcl_DeleteFileHandler';
 %dcl Tk_DeleteTimerHandler char ext;
 %Tk_DeleteTimerHandler='Tcl_DeleteTimerHandler';
 %dcl Tk_DoOneEvent    char ext; %Tk_DoOneEvent  ='Tcl_DoOneEvent';
 %dcl Tk_DoWhenIdle    char ext; %Tk_DoWhenIdle  ='Tcl_DoWhenIdle';
 %dcl Tk_Sleep         char ext; %Tk_Sleep       ='Tcl_Sleep';

 /* Additional stuff that has moved to Tcl: */

 %dcl Tk_AfterCmd      char ext; %Tk_AfterCmd    ='Tcl_AfterCmd';
 %dcl Tk_EventuallyFree char ext;
 %Tk_EventuallyFree='Tcl_EventuallyFree';
 %dcl Tk_FreeProc      char ext; %Tk_FreeProc    ='Tcl_FreeProc';
 %dcl Tk_Preserve      char ext; %Tk_Preserve    ='Tcl_Preserve';
 %dcl Tk_Release       char ext; %Tk_Release     ='Tcl_Release';

 /* Removed Tk_Main, use macro instead */
 Tk_Main: proc (argc, argv, proc)
          options (inline) internal;
   dcl argc fixed bin (31) byvalue,
       argv (*) pointer byaddr, /* each to char (*) varz */
       proc type Tcl_AppInitProc byvalue;
   call Tk_MainEx (argc, argv, proc, Tcl_CreateInterp());
 end Tk_Main;

 %dcl USE_TK_STUBS char ext;
 %if USE_TK_STUBS ^= '' %then %do;
 dcl Tk_InitStubs
     entry (handle Tcl_Interp,
            char (*) varz byaddr,
            fixed bin (31))
     returns (pointer byvalue optional)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tk_InitStubs');
 %end; %else %do;
 Tk_InitStubs: proc (interp, version, exact)
               returns (pointer byvalue optional)
               options (inline) internal;
   dcl interp  handle Tcl_Interp byvalue,
       version char (*) varz byaddr,
       exact   fixed bin (31) byvalue;
   return (Tcl_PkgRequire (interp, 'Tk', version, exact));
 end Tk_InitStubs;
 %end;


 %if USE_TK_STUBS ^= '' & USE_OLD_IMAGE ^= '' %then %do;
 dcl Tk_InitImageArgs
     entry (handle Tcl_Interp,
            fixed bin (31),
            pointer byaddr)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tk_InitImageArgs');
 %end; %else %do;
 Tk_InitImageArgs: proc (interp, argc, argv) options (inline) internal;
   dcl interp handle Tcl_Interp,
       argc   fixed bin (31),
       argv   pointer byaddr;
 end Tk_InitImageArgs;
 %end;


 /* ----------------------------------------------------------------- */
 /*                                                                   */
 /* Additional procedure types defined by Tk.                         */
 /*                                                                   */
 /* ----------------------------------------------------------------- */

 define alias Tk_ErrorProc
        entry (type   ClientData,
               handle XErrorEvent)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_EventProc
        entry (type   ClientData,
               handle XEvent)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_GenericProc
        entry (type   ClientData,
               handle XEvent)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_ClientMessageProc
        entry (type Tk_Window,
               handle XEvent)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_GetSelProc
        entry (type   ClientData,
               handle Tcl_Interp,
               pointer) /* to char (*) varz */
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_LostSelProc
        entry (type   ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_RestrictProc
        entry (type   ClientData,
               handle XEvent)
        returns (type Tk_RestrictAction byvalue)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tk_SelectionProc
        entry (type   ClientData,
               fixed bin (31),
               pointer, /* to char (*) varz */
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 /* ----------------------------------------------------------------- */
 /*                                                                   */
 /* Exported procedures and variables.                                */
 /*                                                                   */
 /* ----------------------------------------------------------------- */

 %include tkDecls;

 %end; /* #TK */
