 /*===================================================================*/
 /*  userenv.hcpy -  Header file for user environment API.            */
 /*                  User Profiles, environment variables,            */
 /*                  and Group Policy                                 */
 /*                                                                   */
 /*  Copyright (c) Microsoft Corporation 1995-1999                    */
 /*  All rights reserved                                              */
 /*                                                                   */
 /*===================================================================*/


 %dcl #INC_USERENV char ext;
 %if #INC_USERENV = '' %then %do;
 %#INC_USERENV = 'Y';

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include windef;
 %end;

 %dcl UNICODE char ext;

 /* Define API decoration for direct importing of DLL references. */
 %dcl USERENVAPI char ext;
 %USERENVAPI = '';

 /*===================================================================*/
 /*                                                                   */
 /* LoadUserProfile                                                   */
 /*                                                                   */
 /* Loads the specified user's profile.                               */
 /*                                                                   */
 /* Most applications should not need to use this function.           */
 /* It's used when a user has logged onto the system or a service     */
 /* starts in a named user account.                                   */
 /*                                                                   */
 /* hToken        - Token for the user, returned from LogonUser()     */
 /* lpProfileInfo - Address of a PROFILEINFO structure                */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:  The caller of this function must have admin privileges on  */
 /*        the machine.                                               */
 /*                                                                   */
 /*        Upon successful return, the hProfile member of the         */
 /*        PROFILEINFO structure is a registry key handle opened to   */
 /*        the root of the user's hive. It has been opened with full  */
 /*        access. If you need to read or write to the user's         */
 /*        registry file, use this key instead of HKEY_CURRENT_USER.  */
 /*        Do not close this handle.                                  */
 /*        Instead pass it to UnloadUserProfile to close the handle.  */
 /*                                                                   */
 /*===================================================================*/

 /* Flags that can be set in the dwFlags field */

 dcl PI_NOUI        type DWORD value('00000001'xu);
     /* Prevents displaying of messages */
 dcl PI_APPLYPOLICY type DWORD value('00000002'xu);
     /* Apply NT4 style policy */

 define structure
   1 PROFILEINFOA,
     2 dwSize        type DWORD,
         /* Set to sizeof(PROFILEINFO) before calling */
     2 dwFlags       type DWORD, /* See flags above */
     2 lpUserName    type LPSTR, /* User name (required) */
     2 lpProfilePath type LPSTR,
         /* Roaming profile path (optional, can be NULL) */
     2 lpDefaultPath type LPSTR,
         /* Default user profile path (optional, can be NULL) */
     2 lpServerName  type LPSTR,
         /* Validating domain controller name in netbios format */
         /* (optional, can be NULL but group NT4 style policy   */
         /* won't be applied)                                   */
     2 lpPolicyPath  type LPSTR,
         /* Path to the NT4 style policy file (optional, can be NULL) */
     2 hProfile      type HANDLE; /* Filled in by the function. */
         /* Registry key handle open to the root. */
 define alias LPPROFILEINFOA handle PROFILEINFOA;

 define structure
   1 PROFILEINFOW,
     2 dwSize        type DWORD,
         /* Set to sizeof(PROFILEINFO) before calling */
     2 dwFlags       type DWORD, /* See flags above */
     2 lpUserName    type LPwSTR, /* User name (required) */
     2 lpProfilePath type LPwSTR,
         /* Roaming profile path (optional, can be NULL) */
     2 lpDefaultPath type LPwSTR,
         /* Default user profile path (optional, can be NULL) */
     2 lpServerName  type LPwSTR,
         /* Validating domain controller name in netbios format */
         /* (optional, can be NULL but group NT4 style policy   */
         /* won't be applied)                                   */
     2 lpPolicyPath  type LPSTR,
         /* Path to the NT4 style policy file (optional, can be NULL) */
     2 hProfile      type HANDLE; /* Filled in by the function. */
         /* Registry key handle open to the root. */
 define alias LPPROFILEINFOW handle PROFILEINFOW;

 %if UNICODE ^= '' %then %do;
 define alias   PROFILEINFO type   PROFILEINFOW;
 define alias LPPROFILEINFO type LPPROFILEINFOW;
 %end; %else %do;
 define alias   PROFILEINFO type   PROFILEINFOA;
 define alias LPPROFILEINFO type LPPROFILEINFOA;
 %end; /* UNICODE */



 dcl LoadUserProfileA
     entry (type HANDLE,
            type PROFILEINFOA byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('LoadUserProfileA');
 dcl LoadUserProfileW
     entry (type HANDLE,
            type PROFILEINFOW byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('LoadUserProfileW');

 %dcl LoadUserProfile char ext;
 %if UNICODE ^= '' %then %do;
   %LoadUserProfile='LoadUserProfileW';
 %end; %else %do;
   %LoadUserProfile='LoadUserProfileA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* UnloadUserProfile                                                 */
 /*                                                                   */
 /* Unloads a user's profile that was loaded by LoadUserProfile()     */
 /*                                                                   */
 /* hToken        -  Token for the user, returned from LogonUser()    */
 /* hProfile      -  hProfile member of the PROFILEINFO structure     */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     The caller of this function must have admin privileges  */
 /*           on the machine.                                         */
 /*                                                                   */
 /*===================================================================*/

 dcl UnloadUserProfile
     entry (type HANDLE,
            type HANDLE)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('UnloadUserProfile');


 /*===================================================================*/
 /*                                                                   */
 /* GetProfilesDirectory                                              */
 /*                                                                   */
 /* Returns the path to the root of where all user profiles are       */
 /* stored.                                                           */
 /*                                                                   */
 /* lpProfilesDir  -  Receives the path                               */
 /* lpcchSize      -  Size of lpProfilesDir                           */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If lpProfilesDir is not large enough,                   */
 /*           the function will fail, and lpcchSize will contain the  */
 /*           necessary buffer size.                                  */
 /*                                                                   */
 /* Example return value: C:\Documents and Settings                   */
 /*                                                                   */
 /*===================================================================*/

 dcl GetProfilesDirectoryA
     entry (char(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetProfilesDirectoryA');
 dcl GetProfilesDirectoryW
     entry (WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetProfilesDirectoryW');

 %dcl GetProfilesDirectory char ext;
 %if UNICODE ^= '' %then %do;
   %GetProfilesDirectory='GetProfilesDirectoryW';
 %end; %else %do;
   %GetProfilesDirectory='GetProfilesDirectoryA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /*  GetProfileType()                                                 */
 /*                                                                   */
 /*  Returns the type of the profile that is loaded for a user.       */
 /*                                                                   */
 /*  dwFlags   - Returns the profile flags                            */
 /*                                                                   */
 /*  Return:     TRUE if successful                                   */
 /*              FALSE if an error occurs.                            */
 /*              Call GetLastError for more details                   */
 /*                                                                   */
 /*  Comments:   if profile is not already loaded the function will   */
 /*              return an error. The caller needs to have access to  */
 /*              HKLM part of the registry. (exists by default)       */
 /*                                                                   */
 /*===================================================================*/

 /* Flags that can be set in the dwFlags field */

 dcl PT_TEMPORARY type DWORD value('00000001'xu);
     /* A profile has been allocated that will be deleted at logoff. */
 dcl PT_ROAMING   type DWORD value('00000002'xu);
     /* The loaded profile is a roaming profile. */
 dcl PT_MANDATORY type DWORD value('00000004'xu);
     /* The loaded profile is mandatory. */

 dcl GetProfileType
     entry (type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetProfileType');


 /*===================================================================*/
 /*                                                                   */
 /* GetDefaultUserProfilesDirectory                                   */
 /*                                                                   */
 /* Returns the path to the root of the default user profile          */
 /*                                                                   */
 /* lpProfileDir   -  Receives the path                               */
 /* lpcchSize      -  Size of lpProfileDir                            */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If lpProfilesDir is not large enough,                   */
 /*           the function will fail, and lpcchSize will contain the  */
 /*           necessary buffer size.                                  */
 /*                                                                   */
 /* Example return value: C:\Documents and Settings\Default User      */
 /*                                                                   */
 /*===================================================================*/

 dcl GetDefaultUserProfileDirectoryA
     entry (char (*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetDefaultUserProfileDirectoryA');
 dcl GetDefaultUserProfileDirectoryW
     entry (WCHAR (*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetDefaultUserProfileDirectoryW');

 %dcl GetDefaultUserProfileDirectory char ext;
 %if UNICODE ^= '' %then %do;
   %GetDefaultUserProfileDirectory='GetDefaultUserProfileDirectoryW';
 %end; %else %do;
   %GetDefaultUserProfileDirectory='GetDefaultUserProfileDirectoryA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* GetAllUsersProfilesDirectory                                      */
 /*                                                                   */
 /* Returns the path to the root of the All Users profile             */
 /*                                                                   */
 /* lpProfileDir   -  Receives the path                               */
 /* lpcchSize      -  Size of lpProfileDir                            */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If lpProfilesDir is not large enough,                   */
 /*           the function will fail, and lpcchSize will contain the  */
 /*           necessary buffer size.                                  */
 /*                                                                   */
 /* Example return value: C:\Documents and Settings\All Users         */
 /*                                                                   */
 /*===================================================================*/

 dcl GetAllUsersProfileDirectoryA
     entry (char(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetAllUsersProfileDirectoryA');
 dcl GetAllUsersProfileDirectoryW
     entry (WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetAllUsersProfileDirectoryW');

 %dcl GetAllUsersProfileDirectory char ext;
 %if UNICODE ^= '' %then %do;
   %GetAllUsersProfileDirectory='GetAllUsersProfileDirectoryW';
 %end; %else %do;
   %GetAllUsersProfileDirectory='GetAllUsersProfileDirectoryA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* GetUserProfileDirectory                                           */
 /*                                                                   */
 /* Returns the path to the root of the requested user's profile      */
 /*                                                                   */
 /* hToken         -  User's token returned from LogonUser()          */
 /* lpProfileDir   -  Receives the path                               */
 /* lpcchSize      -  Size of lpProfileDir                            */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If lpProfileDir is not large enough,                    */
 /*           the function will fail, and lpcchSize will contain the  */
 /*           necessary buffer size.                                  */
 /*                                                                   */
 /* Example return value: C:\Documents and Settings\Joe               */
 /*                                                                   */
 /*===================================================================*/

 dcl GetUserProfileDirectoryA
     entry (type HANDLE,
            char(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetUserProfileDirectoryA');
 dcl GetUserProfileDirectoryW
     entry (type HANDLE,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetUserProfileDirectoryW');

 %dcl GetUserProfileDirectory char ext;
 %if UNICODE ^= '' %then %do;
   %GetUserProfileDirectory='GetUserProfileDirectoryW';
 %end; %else %do;
   %GetUserProfileDirectory='GetUserProfileDirectoryA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* CreateEnvironmentBlock                                            */
 /*                                                                   */
 /* Returns the environment variables for the specified user.         */
 /* This block can then be passed to CreateProcessAsUser().           */
 /*                                                                   */
 /* lpEnvironment  -  Receives a pointer to the new environment block */
 /* hToken         -  User's token returned from LogonUser()          */
 /*                   (optional, can be NULL)                         */
 /* bInherit       -  Inherit from the current process's environment  */
 /*                   block or start from a clean state.              */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If hToken is NULL, the returned environment block will  */
 /*           contain system variables only.                          */
 /*                                                                   */
 /*           Call DestroyEnvironmentBlock to free the buffer when    */
 /*           finished.                                               */
 /*                                                                   */
 /*           If this block is passed to CreateProcessAsUser, the     */
 /*           CREATE_UNICODE_ENVIRONMENT flag must also be set.       */
 /*                                                                   */
 /*===================================================================*/

 dcl CreateEnvironmentBlock
     entry (type LPVOID byaddr,
            type HANDLE,
            type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('CreateEnvironmentBlock');


 /*===================================================================*/
 /*                                                                   */
 /* DestroyEnvironmentBlock                                           */
 /*                                                                   */
 /* Frees environment variables created by CreateEnvironmentBlock     */
 /*                                                                   */
 /* lpEnvironment  -  A pointer to the environment block              */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /*===================================================================*/

 dcl DestroyEnvironmentBlock
     entry (type LPVOID)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('DestroyEnvironmentBlock');


 /*===================================================================*/
 /*                                                                   */
 /* ExpandEnvironmentStringsForUser                                   */
 /*                                                                   */
 /* Expands the source string using the environment block for the     */
 /* specified user.  If hToken is null, the system environment block  */
 /* will be used (no user environment variables).                     */
 /*                                                                   */
 /* hToken         -  User's token returned from LogonUser()          */
 /*                   (optional, can be NULL)                         */
 /* lpSrc          -  Pointer to the string with environment variables*/
 /* lpDest         -  Buffer that receives the expanded string        */
 /* dwSize         -  Size of lpDest in characters (max chars)        */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:     If the user profile for hToken is not loaded, this api  */
 /*           will fail.                                              */
 /*                                                                   */
 /*===================================================================*/

 dcl ExpandEnvironmentStringsForUserA
     entry (type HANDLE,
            char(*) varz byaddr nonasgn,
            char(*) varz byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('ExpandEnvironmentStringsForUserA');
 dcl ExpandEnvironmentStringsForUserW
     entry (type HANDLE,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('ExpandEnvironmentStringsForUserW');

 %dcl ExpandEnvironmentStringsForUser char ext;
 %if UNICODE ^= '' %then %do;
   %ExpandEnvironmentStringsForUser='ExpandEnvironmentStringsForUserW';
 %end; %else %do;
   %ExpandEnvironmentStringsForUser='ExpandEnvironmentStringsForUserA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* RefreshPolicy()                                                   */
 /*                                                                   */
 /* Causes group policy to be applied immediately on the client       */
 /* machine                                                           */
 /*                                                                   */
 /* bMachine  -  Refresh machine or user policy                       */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /*===================================================================*/

 dcl RefreshPolicy
     entry (type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('RefreshPolicy');


 /*===================================================================*/
 /*                                                                   */
 /* EnterCriticalPolicySection                                        */
 /*                                                                   */
 /* Pauses the background application of group policy to allow safe   */
 /* reading of the registry.  Applications that need to read multiple */
 /* policy entries and ensure that the values are not changed while   */
 /* reading them should use this function.                            */
 /*                                                                   */
 /* The maximum amount of time an application can hold a critical     */
 /* section is 10 minutes.                                            */
 /* After 10 minutes, policy can be applied again.                    */
 /*                                                                   */
 /* bMachine -  Pause machine or user policy                          */
 /*                                                                   */
 /* Returns:  Handle if successful                                    */
 /*           NULL if not.  Call GetLastError() for more details      */
 /*                                                                   */
 /* Note 1:  The handle returned should be passed to                  */
 /*          LeaveCriticalPolicy Section when finished.  Do not close */
 /*          this handle, LeaveCriticalPolicySection will do that.    */
 /*                                                                   */
 /* Note 2:  If both user and machine critical sections need to be    */
 /*          acquired then they should be done in this order:         */
 /*          first acquire user critical section and                  */
 /*          then acquire machine critical section.                   */
 /*                                                                   */
 /*===================================================================*/

 dcl EnterCriticalPolicySection
     entry (type BOOL)
     returns (type HANDLE byvalue optional)
     WINAPI USERENVAPI external('EnterCriticalPolicySection');


 /*===================================================================*/
 /*                                                                   */
 /* LeaveCriticalPolicySection                                        */
 /*                                                                   */
 /* Resumes the background application of group policy.               */
 /* See EnterCriticalPolicySection for more details.                  */
 /*                                                                   */
 /* hSection - Handle returned from EnterCriticalPolicySection        */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Call GetLastError() for more details     */
 /*                                                                   */
 /* Note:  This function will close the handle.                       */
 /*                                                                   */
 /*===================================================================*/

 dcl LeaveCriticalPolicySection
     entry (type HANDLE)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('LeaveCriticalPolicySection');


 /*===================================================================*/
 /*                                                                   */
 /* RegisterGPNotification                                            */
 /*                                                                   */
 /* Entry point for registering for Group Policy change notification. */
 /*                                                                   */
 /* Parameters: hEvent     -   Event to be notified, by calling       */
 /*                            SetEvent (hEvent)                      */
 /*             bMachine   -   If true, then register machine policy  */
 /*                            notification                           */
 /*                            else register user policy notification */
 /*                                                                   */
 /* Returns:    True if successful                                    */
 /*             False if error occurs                                 */
 /*                                                                   */
 /* Notes:      Group Policy Notifications.  There are 2 ways an      */
 /*             application can be notified when Group Policy is      */
 /*             finished being applied.                               */
 /*                                                                   */
 /*             1) Using the RegisterGPNotifcation function and       */
 /*                waiting for the event to be signalled.             */
 /*                                                                   */
 /*             2) A WM_SETTINGCHANGE message is broadcast to all     */
 /*                desktops.                                          */
 /*                wParam - 1 if machine policy was applied,          */
 /*                         0 if user policy was applied.             */
 /*                lParam - Points to the string "Policy"             */
 /*                                                                   */
 /*===================================================================*/

 dcl RegisterGPNotification
     entry (type HANDLE,
            type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('RegisterGPNotification');


 /*===================================================================*/
 /*                                                                   */
 /* UnregisterGPNotification                                          */
 /*                                                                   */
 /* Removes registration for a Group Policy change notification.      */
 /*                                                                   */
 /* Parameters: hEvent    -   Event to be removed                     */
 /*                                                                   */
 /* Returns:    True if successful                                    */
 /*             False if error occurs                                 */
 /*                                                                   */
 /*===================================================================*/

 dcl UnregisterGPNotification
     entry (type HANDLE)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('UnregisterGPNotification');


 /*===================================================================*/
 /*                                                                   */
 /* GPOptions flags                                                   */
 /*                                                                   */
 /* These are the flags found in the GPOptions property of a DS object*/
 /*                                                                   */
 /* For a given DS object (Site, Domain, OU), the GPOptions property  */
 /* contains options that effect all the GPOs link to this SDOU.      */
 /*                                                                   */
 /* This is a DWORD type                                              */
 /*                                                                   */
 /*===================================================================*/

 dcl GPC_BLOCK_POLICY type DWORD value('00000001'xu);
     /* Block all non-forced policy from above */


 /*===================================================================*/
 /*                                                                   */
 /* GPLink flags                                                      */
 /*                                                                   */
 /* These are the flags found on the GPLink property of a DS object   */
 /* after the GPO path.                                               */
 /*                                                                   */
 /* For a given DS object (Site, Domain, OU), the GPLink property     */
 /* will be in this text format                                       */
 /*                                                                   */
 /* [LDAP:/*CN={E615A0E3-C4F1-11D1-A3A7-00AA00615092},CN=Policies,    */
 /*         CN=System,DC=ntdev,DC=Microsoft,DC=Com;1]                 */
 /*                                                                   */
 /* The GUID is the GPO name, and the number following the LDAP path  */
 /* are the options for that link from this DS object.                */
 /* Note, there can be multiple GPOs                                  */
 /* each in their own square brackets in a prioritized list.          */
 /*                                                                   */
 /*===================================================================*/

 /* Options for a GPO link */

 dcl GPO_FLAG_DISABLE type DWORD value('00000001'xu);
     /* This GPO is disabled */
 dcl GPO_FLAG_FORCE   type DWORD value('00000002'xu);
     /* Don't override the settings in this GPO with settings from */
     /* a GPO below it. */


 /*===================================================================*/
 /*                                                                   */
 /* GetGPOList                                                        */
 /*                                                                   */
 /*                                                                   */
 /* Queries for the list of Group Policy Objects for the specified    */
 /* user or machine.  This function will return a link list           */
 /* of Group Policy Objects.  Call FreeGPOList to free the list.      */
 /*                                                                   */
 /* Note, most applications will not need to call this function.      */
 /* This will primarily be used by services acting on behalf of       */
 /* another user or machine.  The caller of this function will        */
 /* need to look in each GPO for their specific policy                */
 /*                                                                   */
 /* This function can be called two different ways.                   */
 /* Either the hToken for a user or machine can be supplied and the   */
 /* correct name and domain controller name will be generated,        */
 /* or hToken is NULL and the caller must supply the name and the     */
 /* domain controller name.                                           */
 /*                                                                   */
 /* Calling this function with an hToken ensures the list of Group    */
 /* Policy Objects is correct for the user or machine since security  */
 /* access checking can be perfomed.  If hToken is not supplied,      */
 /* the security of the caller is used instead which means that list  */
 /* may or may not be 100% correct for the intended user / machine.   */
 /* However, this is the fastest way to call this function.           */
 /*                                                                   */
 /* hToken           - User or machine token, if NULL, lpName and     */
 /*                    lpHostName must be supplied                    */
 /* lpName           - User or machine name in DN format,             */
 /*                    if hToken is supplied, this must be NULL       */
 /* lpHostName       - Domain DN name or domain controller name.      */
 /*                    If hToken is supplied, this must be NULL       */
 /* lpComputerName   - Computer name to use to determine site         */
 /*                    location. If NULL, the local computer is used  */
 /*                    as the reference. Format:  \\machinename       */
 /* dwFlags          - Flags field.  See flags definition below       */
 /* pGPOList         - Address of a pointer which receives the link   */
 /*                    list of GPOs                                   */
 /*                                                                   */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not.  Use GetLastError() for more details.     */
 /*                                                                   */
 /* Examples:                                                         */
 /*                                                                   */
 /* Here's how this function will typically be called for             */
 /* looking up the list of GPOs for a user:                           */
 /*                                                                   */
 /*      dcl GPOList type GROUP_POLICY_OBJECT                         */
 /*                                                                   */
 /*      if GetGPOList (hToken, *, *, *, 0, GPOList) = TRUE then     */
 /*      do                                                           */
 /*          /- do processing here... -/                              */
 /*          call FreeGPOList (pGPOList)                              */
 /*      end                                                          */
 /*                                                                   */
 /*                                                                   */
 /* Here's how this function will typically be called for             */
 /* looking up the list of GPOs for a machine:                        */
 /*                                                                   */
 /*      dcl GPOList type GROUP_POLICY_OBJECT                         */
 /*                                                                   */
 /*      if GetGPOList (sysnull(), lpMachineName, lpHostName,         */
 /*                     lpMachineName,                                */
 /*                     GPO_LIST_FLAG_MACHINE, GPOList) = TRUE then   */
 /*      do                                                           */
 /*          /- do processing here... -/                              */
 /*          call FreeGPOList (pGPOList)                              */
 /*      end                                                          */
 /*                                                                   */
 /*===================================================================*/

 /* Each Group Policy Object is associated (linked) with a site,      */
 /* domain, organizational unit, or machine.                          */

 define ordinal GPO_LINK
   ( GPLinkUnknown value(0),  /* No link information available */
     GPLinkMachine,      /* GPO linked to a machine (local or remote) */
     GPLinkSite,              /* GPO linked to a site */
     GPLinkDomain,            /* GPO linked to a domain */
     GPLinkOrganizationalUnit /* GPO linked to a organizational unit */
   ) prec(31);
 dcl PGPO_LINK pointer;

 define structure
   1 GROUP_POLICY_OBJECTA,
    2 dwOptions     type DWORD,    /* See GPLink option flags above */
    2 dwVersion     type DWORD,    /* Revision number of the GPO */
    2 lpDSPath      type LPSTR,
                    /* Path to the Active Directory portion of the GPO*/
    2 lpFileSysPath type LPSTR,
                    /* Path to the file system portion of the GPO */
    2 lpDisplayName type LPSTR,    /* Friendly display name */
    2 szGPOName     char (49) varz,/* Unique name */
    2 GPOLink       type GPO_LINK, /* Link information */
    2 lParam        type LPARAM,
      /* Free space for the caller to store GPO specific information */
    2 pNext         handle GROUP_POLICY_OBJECTA,
                                   /* Next GPO in the list */
    2 pPrev         handle GROUP_POLICY_OBJECTA,
                                   /* Previous GPO in the list */
    2 lpExtensions  type LPSTR,
                    /* Extensions that are relevant for this GPO */
    2 lParam2       type LPARAM;
      /* Free space for the caller to store GPO specific information */
 define alias PGROUP_POLICY_OBJECTA handle GROUP_POLICY_OBJECTA;

 define structure
   1 GROUP_POLICY_OBJECTW,
    2 dwOptions     type DWORD,    /* See GPLink option flags above */
    2 dwVersion     type DWORD,    /* Revision number of the GPO */
    2 lpDSPath      type LPWSTR,
                    /* Path to the Active Directory portion of the GPO*/
    2 lpFileSysPath type LPWSTR,
                    /* Path to the file system portion of the GPO */
    2 lpDisplayName type LPWSTR,    /* Friendly display name */
    2 szGPOName     WCHAR (49) varz,/* Unique name */
    2 GPOLink       type GPO_LINK,  /* Link information */
    2 lParam        type LPARAM,
      /* Free space for the caller to store GPO specific information */
    2 pNext         handle GROUP_POLICY_OBJECTW,
                                    /* Next GPO in the list */
    2 pPrev         handle GROUP_POLICY_OBJECTW,
                                    /* Previous GPO in the list */
    2 lpExtensions  type LPWSTR,
                    /* Extensions that are relevant for this GPO */
    2 lParam2       type LPARAM;
      /* Free space for the caller to store GPO specific information */
 define alias PGROUP_POLICY_OBJECTW handle GROUP_POLICY_OBJECTW;

 %if UNICODE ^= '' %then %do;
 define alias  GROUP_POLICY_OBJECT type  GROUP_POLICY_OBJECTW;
 define alias PGROUP_POLICY_OBJECT type PGROUP_POLICY_OBJECTW;
 %end; %else %do;
 define alias  GROUP_POLICY_OBJECT type  GROUP_POLICY_OBJECTA;
 define alias PGROUP_POLICY_OBJECT type PGROUP_POLICY_OBJECTA;
 %end; /* UNICODE */


 /* dwFlags for GetGPOList() */

 dcl GPO_LIST_FLAG_MACHINE  type DWORD value('00000001'xu);
     /* Return machine policy information */
 dcl GPO_LIST_FLAG_SITEONLY type DWORD value('00000002'xu);
     /* Return site policy information only */


 dcl GetGPOListA
     entry (type HANDLE,
            char(*) varz byaddr nonasgn optional,
            char(*) varz byaddr nonasgn optional,
            char(*) varz byaddr nonasgn optional,
            type DWORD,
            type PGROUP_POLICY_OBJECTA byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetGPOListA');
 dcl GetGPOListW
     entry (type HANDLE,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            type PGROUP_POLICY_OBJECTW byaddr)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('GetGPOListW');

 %dcl GetGPOList char ext;
 %if UNICODE ^= '' %then %do;
   %GetGPOList='GetGPOListW';
 %end; %else %do;
   %GetGPOList='GetGPOListA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* FreeGPOList                                                       */
 /*                                                                   */
 /*                                                                   */
 /* Frees the link list returned from GetGPOList                      */
 /*                                                                   */
 /* pGPOList - Pointer to the link list of GPOs                       */
 /*                                                                   */
 /*                                                                   */
 /* Returns:  TRUE if successful                                      */
 /*           FALSE if not                                            */
 /*                                                                   */
 /*===================================================================*/

 dcl FreeGPOListA
     entry (type PGROUP_POLICY_OBJECTA)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('FreeGPOListA');
 dcl FreeGPOListW
     entry (type PGROUP_POLICY_OBJECTW)
     returns (type BOOL byvalue optional)
     WINAPI USERENVAPI external('FreeGPOListW');
 %dcl FreeGPOList char ext;
 %if UNICODE ^= '' %then %do;
   %FreeGPOList='FreeGPOListW';
 %end; %else %do;
   %FreeGPOList='FreeGPOListA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* GetAppliedGPOList                                                 */
 /*                                                                   */
 /* Queries for the list of applied Group Policy Objects for the      */
 /* specified user or machine and specified client side extension.    */
 /* This function will return a linked listof Group Policy Objects.   */
 /* Call FreeGPOList to free the list.                                */
 /*                                                                   */
 /* dwFlags          - User or machine policy, if it is               */
 /*                    GPO_LIST_FLAG_MACHINE then                     */
 /*                    return machine policy information              */
 /* pMachineName     - Name of remote computer in the form            */
 /*                    \\computername.                                */
 /*                    If null then local computer is used.           */
 /* pSidUser         - Security id of user (relevant for user policy).*/
 /*                    If pMachineName is null and pSidUser is null   */
 /*                    then it means current logged on user.          */
 /*                    If pMachine is null and pSidUser is non-null   */
 /*                    then it means user represented by pSidUser on  */
 /*                    local machine. If pMachineName is non-null     */
 /*                    then and if dwFlags specifies user policy,     */
 /*                    then pSidUser must be non-null.                */
 /* pGuidExtension   - Guid of the specified extension                */
 /* ppGPOList        - Address of a pointer which receives the link   */
 /*                    list of GPOs                                   */
 /*                                                                   */
 /* The return value is a Win32 error code. ERROR_SUCCESS means the   */
 /* GetAppliedGPOList function completed successfully.                */
 /* Otherwise it indicates that the function failed.                  */
 /*                                                                   */
 /*===================================================================*/

 dcl GetAppliedGPOListA
     entry (type DWORD,
            char(*) varz byaddr nonasgn optional,
            type PSID,
            type GUID byaddr,
            type PGROUP_POLICY_OBJECTA byaddr)
     returns (type DWORD byvalue optional)
     WINAPI USERENVAPI external('GetAppliedGPOListA');
 dcl GetAppliedGPOListW
     entry (type DWORD,
            WCHAR(*) varz byaddr nonasgn optional,
            type PSID,
            type GUID byaddr,
            type PGROUP_POLICY_OBJECTW byaddr)
     returns (type DWORD byvalue optional)
     WINAPI USERENVAPI external('GetAppliedGPOListW');

 %dcl GetAppliedGPOList char ext;
 %if UNICODE ^= '' %then %do;
   %GetAppliedGPOList='GetAppliedGPOListW';
 %end; %else %do;
   %GetAppliedGPOList='GetAppliedGPOListA';
 %end; /* !UNICODE */


 /*===================================================================*/
 /*                                                                   */
 /* Group Policy Object client side extension support                 */
 /*                                                                   */
 /* Flags, data structures and function prototype                     */
 /*                                                                   */
 /* To register your extension, create a subkey under this key        */
 /*                                                                   */
 /* Software\Microsoft\Windows NT\CurrentVersion\Winlogon\GPExtensions*/
 /*                                                                   */
 /* The subkey needs to be a guid so that it is unique. The noname    */
 /* value of the subkey can be the friendly name of the extension.    */
 /* Then add these values:                                            */
 /*                                                                   */
 /*     DllName                      REG_EXPAND_SZ                    */
 /*        Path to your DLL                                           */
 /*     ProcessGroupPolicy           REG_SZ                           */
 /*        Function name (see PFNPROCESSGROUPPOLICY prototype)        */
 /*     NoMachinePolicy              REG_DWORD                        */
 /*        True, if extension does not have to be called when         */
 /*        machine policies are being processed.                      */
 /*     NoUserPolicy                 REG_DWORD                        */
 /*        True, if extension does not have to be called when         */
 /*        user policies are being processed.                         */
 /*     NoSlowLink                   REG_DWORD                        */
 /*        True, if extension does not have to be called on a slow    */
 /*        link                                                       */
 /*     NoBackgroundPolicy           REG_DWORD                        */
 /*        True, if extension does not have to be called on.          */
 /*        policies applied in background.                            */
 /*     NoGPOListChanges             REG_DWORD                        */
 /*        True, if extension does not have to be called when         */
 /*        there are no changes between cached can current GPO lists. */
 /*     PerUserLocalSettings         REG_DWORD                        */
 /*        True, if user policies have to be cached on a per user and */
 /*        per machine basis.                                         */
 /*     RequiresSuccessfulRegistry   REG_DWORD                        */
 /*        True, if extension should be called only if registry       */
 /*        extension was successfully processed.                      */
 /*     EnableAsynchronousProcessing REG_DWORD                        */
 /*        True, if registry extension will complete its processing   */
 /*        asynchronously.                                            */
 /*     NotifyLinkTransition         REG_DWORD                        */
 /*        True, if extension should be called when a change in link  */
 /*        speed is detected between previous policy application and  */
 /*        current policy application.                                */
 /*                                                                   */
 /* The return value is a Win32 error code. ERROR_SUCCESS means the   */
 /* ProcessGroupPolicy function completed successfully.               */
 /* If return value is ERROR_OVERRIDE_NOCHANGES then it means that the*/
 /* extension will be called the next time even if NoGPOListChanges   */
 /* is set and there are no changes to the GPO list. Any other return */
 /* value indicates that the ProcessGroupPolicy function failed.      */
 /*                                                                   */
 /*===================================================================*/

 dcl GPO_INFO_FLAG_MACHINE            type DWORD value('00000001'xu);
     /* Apply machine policy rather than user policy */
 dcl GPO_INFO_FLAG_BACKGROUND         type DWORD value('00000010'xu);
     /* Background refresh of policy (ok to do slow stuff) */
 dcl GPO_INFO_FLAG_SLOWLINK           type DWORD value('00000020'xu);
     /* Policy is being applied across a slow link */
 dcl GPO_INFO_FLAG_VERBOSE            type DWORD value('00000040'xu);
     /* Verbose output to the eventlog */
 dcl GPO_INFO_FLAG_NOCHANGES          type DWORD value('00000080'xu);
     /* No changes were detected to the Group Policy Objects */
 dcl GPO_INFO_FLAG_LINKTRANSITION     type DWORD value('00000100'xu);
     /* A change in link speed was detected between previous policy */
     /* application and current policy application */

 define alias ASYNCCOMPLETIONHANDLE type UINT_PTR;

 /* Original C declarations do not give linkage convention. */
 /* I have assumed that it is system standard CALLBACK.     */
 define alias PFNSTATUSMESSAGECALLBACK
        entry (type BOOL,
               type LPWSTR)
        returns (type DWORD byvalue optional)
        limited CALLBACK;

 define alias PFNPROCESSGROUPPOLICY
        entry (type DWORD,             /* GPO_INFO_FLAGS              */
               type HANDLE,            /* User or machine token       */
               type HKEY,              /* Root of registry            */
               type PGROUP_POLICY_OBJECT,
                                       /* Linked list of deleted GPOs */
               type PGROUP_POLICY_OBJECT,
                                       /* Linked list of changed GPOs */
               type ASYNCCOMPLETIONHANDLE,
                                       /* For asynchronous completion */
               type BOOL byaddr,/* If true, then abort GPO processing */
               type PFNSTATUSMESSAGECALLBACK)
                  /* Callback function for displaying status messages */
                  /* Note, this can be NULL */
        returns (type DWORD byvalue optional)
        limited CALLBACK;

 /* GUID that identifies the registry extension */

 dcl 1 REGISTRY_EXTENSION_GUID static,         /* type GUID */
       2 Data1 type DWORD init ('35378EAC'xu),
       2 Data2 type WORD  init ('683F'xu),
       2 Data3 type WORD  init ('11D2'xu),
       2 Data4 (8) char   init ('A8'x, '9A'x, '00'x, 'C0'x,
                                '4F'x, 'BB'x, 'CF'x, 'A2'x);


 /*===================================================================*/
 /*                                                                   */
 /* Group Policy Object client side asynchronous extension processing */
 /*                                                                   */
 /* extensionId    - Unique guid identifying the extension            */
 /* pAsyncHandle   - Asynchronous completion handle that was passed   */
 /*                  to extension in ProcessGroupPolicy call          */
 /* dwStatus       - Completion status of asynchronous processing     */
 /*                                                                   */
 /* The return value is a Win32 error code. ERROR_SUCCESS means the   */
 /* ProcessGroupPolicyCompleted function completed successfully.      */
 /* Otherwise it indicates that the function failed.                  */
 /*                                                                   */
 /*===================================================================*/

 define alias REFGPEXTENSIONID handle GUID;

 dcl ProcessGroupPolicyCompleted
     entry (type REFGPEXTENSIONID,
            type ASYNCCOMPLETIONHANDLE,
            type DWORD)
     returns (type DWORD byvalue optional)
     WINAPI USERENVAPI external('ProcessGroupPolicyCompleted');
 %end; /* _INC_USERENV */
