/******************************************************************/
/*                                                                */
/*    Licensed Materials - Property of IBM                        */
/*    5639-A83, 5639-A24 (C) Copyright IBM Corp. 1992,1996.       */
/*    All Rights Reserved.                                        */
/*    US Government Users Restricted Rights-- Use, duplication or */
/*    disclosure restricted by GSA ADP Schedule Contract with     */
/*    IBM Corp.                                                   */
/*                                                                */
/*  DISCLAIMER OF WARRANTIES                                      */
/*    The following enclosed code is sample code created by IBM */
/*    Corporation. This sample code is not part of any standard   */
/*    IBM product and is provided to you solely for the purpose of*/
/*    assisting you in the development of your applications.  The */
/*    code is provided "AS IS", without warranty of any kind.     */
/*    IBM shall not be liable for any damages arising out of your */
/*    use of the sample code, even if IBM has been advised of the */
/*    possibility of such damages.                                */
/*                                                                */
/*                                                                */
/* This is the C Set/2 program used to call the PL/I procedure    */
/* which has been compiled as object and packaged into a DLL.     */
/* The C program will first load the PL/I DLL, query its first    */
/* entry point, then call it along with any necessary arguements. */
/* It will also use printf function to print out message to       */
/* show the progress of each major step during execution.         */
/* At the end, the PL/I DLL will be freed explicitly to ensure    */
/* proper program termination.                                    */
/*                                                                */
/* Syntax:                                                        */
/*    CS2PLI [arg1] [arg2]                                        */
/*           where arg1 is the name of the PL/I DLL               */
/*           where arg2 is anything to be passed to the PL/I DLL. */
/*                                                                */
/* If no argument is supplied, CS2PLI will assume the DLL of      */
/* "HIWORLD" and will call it.                                    */
/*                                                                */
/******************************************************************/
#include <windows.h>
#include <stdio.h>
#include <ctype.h>

int main (int argc, char *argv[])
{
   /* argv is a ptr to an array of char ptrs.  argv[0] = EXE name.  */
   /* argv[1] = 1st cmd line arguement.  argc = 1 means no argument.*/

   HMODULE mh;
   unsigned long handle;
   char pn[]="HIWORLD";
   FARPROC procAddr;
   char *pc, *upc;
   char objnamebuf[16];

   /******************************************************************/
   /* Pick up the first argument from the command line string if any.*/
   /* e.g. CS2PLI arg1 arg2 and arg1 is the first argument to CS2PLI.*/
   /******************************************************************/

   if (argc > 1)
     {
       upc = argv[1];
       while ( (*upc) != '\0')
         {
           (*upc) = toupper( (*upc) );
           upc ++;
         } /* endwhile */
       pc = argv[1];    /* Pick up the optional cmd line argument.*/
     }
   else
     pc = pn;           /* Default is to pick up "HIWORLD".       */

   fprintf(stderr, "The Beginning of CS2PLI.....\n");
   fprintf(stderr,"-- First call to load the PL/I DLL--\n");

   /******************************************************************/
   /* _loadmod loads the PL/I DLL and returns the module             */
   /* handle to be used later.                                       */
   /******************************************************************/

   if (_loadmod(pc,&handle)) {
     printf("Error loading module %s\n",pc);
     return EXIT_FAILURE;
   }
   else
     printf("_loadmod return successfully\n");


   /******************************************************************/
   /* GetProcAddress finds the first entry point in the loaded       */
   /* PL/I DLL and returns its address.                              */
   /******************************************************************/

   procAddr = GetProcAddress((HMODULE) handle, pc);
   if (procAddr == (FARPROC) 0)
    {
     printf("Error loading module %s\n",pc);
     return EXIT_FAILURE;
    }
   else
     printf("GetProcAddress's returned address=%d.\n",procAddr);

   /******************************************************************/
   /* Pick up the second argument from the command line string if    */
   /* any.  e.g. CS2PLI arg1 arg2 and arg2 is the second argument    */
   /* to CS2PLI.                                                     */
   /* Then, call the PL/I procedure using the entry point address    */
   /* returned from GetProcAddress and pass argument to it if any.   */
   /******************************************************************/

   if (argc > 2)
     {
       upc = argv[2];
       while ( (*upc) != '\0')
         {
           (*upc) = toupper( (*upc) );
           upc ++;
         } /* endwhile */
      fprintf(stderr,"The command line arg2 = %s.\n", argv[2]);
      (void (* APIENTRY)(char *))procAddr(argv[2]);
     }
   else
     procAddr();

   /******************************************************************/
   /* Call the same PL/I procedure again and pass the argument to    */
   /* it if any.                                                     */
   /******************************************************************/

   fprintf(stderr,"-- Second call to the routine already loaded --\n");
   if (argc > 2)
     (void (* APIENTRY)(char *))procAddr(argv[2]);
   else
     procAddr();

   fprintf(stderr,"After 2nd call and before _freemod.\n");

   /******************************************************************/
   /* Free the PL/I DLL before terminating this C program.           */
   /******************************************************************/

   if (_freemod(handle)) {
     printf("Error in freeing the %s DLL module\n",pc) ;
     return EXIT_FAILURE;
   }
   else
   fprintf(stderr,"The End of CS2PLI............\n");
   return(0);
}
